@@ -21,6 +21,10 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: common.h,v 1.14 2002/06/13 10:02:51 stamfest Exp $ */
+
+#ifndef GQ_COMMON_H_INCLUDED
+#define GQ_COMMON_H_INCLUDED
 
 #include <lber.h>
 #include <ldap.h>
@@ -65,21 +69,50 @@ struct ldapserver {
      char basedn[MAX_DN_LEN];
      char binddn[MAX_DN_LEN];
      char bindpw[MAX_BINDPW_LEN];
+     /* split the "configuration" password from the one entered by
+	hand. This simplifies the handling of the configured password
+	considerably */
+     char enteredpw[MAX_BINDPW_LEN];
      int bindtype;
      char searchattr[MAX_ATTR_LEN];
      int maxentries;
      int cacheconn;
      int enabletls;
+     long local_cache_timeout;
+     int ask_pw;
+
      LDAP *connection;
+     int missing_closes; /* incremented on every open_connection,
+			    decrementen on each close, close_connection
+			    really closes only if this drops to zero */
      struct server_schema *ss;
      int flags;
 
      struct ldapserver *next;
+     int version;
+     /* server_down is a flag set by the SIGPIPE signal handler and in
+	case of an LDAP_SERVER_DOWN error. It indicates that we should
+	reconnect the next time open_connection gets called. There is
+	no (simple) way to find out which connection has been broken
+	in case of a SIGPIPE, thus we have to record this for every
+	connection. We MIGHT instead check for ld_errno in the LDAP
+	structure, but that is neither really documented (some man
+	pages mention it though) nor is it actually available through
+	ldap.h */
+     int server_down;
 };
 
 
 struct tokenlist {
      int token;
      char keyword[32];
+     gpointer data;
 };
 
+#endif
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
