@@ -21,6 +21,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: configfile.c,v 1.23 2002/07/03 18:49:45 stamfest Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -30,6 +31,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 #include <ctype.h>
+#include <signal.h>
 
 #include <glib.h>
 
@@ -40,6 +42,7 @@
 #include "filter.h"
 #include "errorchain.h"
 #include "debug.h"
+#include "i18n.h"
 
 
 struct gq_config config;
@@ -72,6 +75,8 @@ struct keywordlist configwords[] = {
      { "cache-connection",T_CACHECONN,         NEEDS_CLOSE|NEEDS_DATA },
      { "enable-tls",      T_ENABLETLS,         NEEDS_CLOSE|NEEDS_DATA },
      { "schema-server",   T_SCHEMASERVER,      NEEDS_CLOSE|NEEDS_DATA },
+     { "local-cache-timeout", T_LOCAL_CACHE_TIMEOUT, NEEDS_CLOSE|NEEDS_DATA },
+     { "ask-pw",	  T_ASK_PW,	       NEEDS_CLOSE|NEEDS_DATA },
 
      /* templates */
      { "template",        T_TEMPLATE,          NEEDS_CLOSE },
@@ -97,6 +102,7 @@ struct tokenlist token_searchargument[] = {
 struct tokenlist token_bindtype[] = {
      { BINDTYPE_SIMPLE,    "Simple" },
      { BINDTYPE_KERBEROS,  "Kerberos"},
+     { BINDTYPE_SASL,      "SASL"},
      { 0, "" }
 };
 
@@ -134,6 +140,7 @@ struct gq_template *new_template(void)
      struct gq_template *tmpl;
 
      tmpl = MALLOC(sizeof(struct gq_template), "struct gq_template");
+     memset(tmpl, 0, sizeof(struct gq_template));
      tmpl->name[0] = '\0';
      tmpl->objectclasses = NULL;
 
@@ -149,6 +156,7 @@ struct gq_filter *new_filter(void)
      struct gq_filter *filter;
 
      filter = MALLOC(sizeof(struct gq_filter), "struct gq_filter");
+     memset(filter, 0, sizeof(struct gq_filter));
      filter->name[0] = '\0';
      filter->ldapfilter[0] = '\0';
      filter->servername[0] = '\0';
@@ -236,8 +244,9 @@ int config_get_bool(struct configfile *f)
      else if(!strcasecmp(f->cur_string, "false"))
 	  return(0);
 
-     sprintf(errstr, "line %d: boolean value '%s' should be 'True' "
-	     "or 'False'", f->line, f->cur_string);
+     snprintf(errstr, sizeof(errstr),
+	      _("line %d: boolean value '%s' should be 'True' "
+		"or 'False'"), f->line, f->cur_string);
      error_push(f->err_context, errstr);
 
      return(0);
@@ -339,7 +348,7 @@ void process_rcfile(char *rcfile)
      f->sp = 1;
      f->cur_ldapserver = NULL;
      f->cur_template = NULL;
-     f->err_context = error_new_context("Error parsing configfile");
+     f->err_context = error_new_context(_("Error parsing configfile"));
 
      while(f->rcfile[f->p] != '\0') {
 	  do {
@@ -354,8 +363,9 @@ void process_rcfile(char *rcfile)
 	  if(f->cur_token == T_DATA) {
 	       /* FIXME this is ugly */
 	       if(f->sp < 3) {
-		    sprintf(errstr, "line %d: unmarked data '%s'",
-			    f->line, f->cur_string);
+		    snprintf(errstr, sizeof(errstr), 
+			     _("line %d: unmarked data '%s'"),
+			     f->line, f->cur_string);
 		    error_push(f->err_context, errstr);
 		    break;
 	       }
@@ -395,7 +405,7 @@ void process_rcfile(char *rcfile)
 			 config.ldifformat = tokenize(token_ldifformat, f->cur_string);
 			 break;
 		    case T_SCHEMASERVER:
-			 strncpy(config.schemaserver, f->cur_string, MAX_SERVERNAME_LEN);
+			 strncpy(config.schemaserver, f->cur_string, MAX_SERVERNAME_LEN - 1);
 			 break;
 		    }
 	       }
@@ -405,27 +415,33 @@ void process_rcfile(char *rcfile)
 			 /* filling in an <ldapserver> struct? */
 			 switch(f->stack[f->sp - 1]) {
 			 case T_NAME:
-			      strcpy(f->cur_ldapserver->name, f->cur_string);
+			      strncpy(f->cur_ldapserver->name, f->cur_string,
+				      sizeof(f->cur_ldapserver->name) - 1);
 			      break;
 			 case T_LDAPHOST:
-			      strcpy(f->cur_ldapserver->ldaphost, f->cur_string);
+			      strncpy(f->cur_ldapserver->ldaphost, f->cur_string,
+				     sizeof(f->cur_ldapserver->ldaphost) - 1);
 			      break;
 			 case T_LDAPPORT:
 			      f->cur_ldapserver->ldapport = atoi(f->cur_string);
 			      break;
 			 case T_BASEDN:
-			      strcpy(f->cur_ldapserver->basedn, f->cur_string);
+			      strncpy(f->cur_ldapserver->basedn, f->cur_string,
+				      sizeof(f->cur_ldapserver->basedn) - 1);
 			      break;
 			 case T_BINDDN:
-			      strcpy(f->cur_ldapserver->binddn, f->cur_string);
+			      strncpy(f->cur_ldapserver->binddn, f->cur_string,
+				      sizeof(f->cur_ldapserver->binddn) - 1);
 			      break;
 			 case T_BINDPW:
-			      strcpy(f->cur_ldapserver->bindpw, f->cur_string);
+			      strncpy(f->cur_ldapserver->bindpw, f->cur_string,
+				      sizeof(f->cur_ldapserver->bindpw) - 1);
 			      break;
 			 case T_BINDTYPE:
 			      f->cur_ldapserver->bindtype = tokenize(token_bindtype, f->cur_string);
 			 case T_SEARCHATTR:
-			      strcpy(f->cur_ldapserver->searchattr, f->cur_string);
+			      strncpy(f->cur_ldapserver->searchattr, f->cur_string,
+				      sizeof(f->cur_ldapserver->searchattr) - 1);
 			      break;
 			 case T_MAXENTRIES:
 			      f->cur_ldapserver->maxentries = atoi(f->cur_string);
@@ -435,6 +451,13 @@ void process_rcfile(char *rcfile)
 			      break;
 			 case T_ENABLETLS:
 			      f->cur_ldapserver->enabletls = config_get_bool(f);
+			      break;
+			 case T_LOCAL_CACHE_TIMEOUT:
+			      f->cur_ldapserver->local_cache_timeout = atoi(f->cur_string);
+			      break;
+			 case T_ASK_PW:
+			      f->cur_ldapserver->ask_pw = config_get_bool(f);
+			      break;
 
 			 }
 
@@ -443,7 +466,8 @@ void process_rcfile(char *rcfile)
 			 /* fill in gq_template struct */
 			 switch(f->stack[f->sp - 1]) {
 			 case T_NAME:
-			      strncpy(f->cur_template->name, f->cur_string, 128);
+			      strncpy(f->cur_template->name, f->cur_string, 
+				      sizeof(f->cur_template->name) - 1);
 			      break;
 			 case T_OBJECTCLASS:
 			      f->cur_template->objectclasses =
@@ -454,16 +478,20 @@ void process_rcfile(char *rcfile)
 		    else if(f->sp > 2 && f->stack[f->sp - 2] == T_FILTER) {
 			 switch(f->stack[f->sp - 1]) {
 			 case T_NAME:
-			      strncpy(f->cur_filter->name, f->cur_string, 127);
+			      strncpy(f->cur_filter->name, f->cur_string, 
+				      sizeof(f->cur_filter->name) - 1);
 			      break;
 			 case T_LDAPFILTER:
-			      strncpy(f->cur_filter->ldapfilter, f->cur_string, 1023);
+			      strncpy(f->cur_filter->ldapfilter, f->cur_string,
+				      sizeof(f->cur_filter->ldapfilter) - 1);
 			      break;
 			 case T_SERVERNAME:
-			      strncpy(f->cur_filter->servername, f->cur_string, MAX_SERVERNAME_LEN - 1);
+			      strncpy(f->cur_filter->servername, f->cur_string,
+				      MAX_SERVERNAME_LEN - 1);
 			      break;
 			 case T_BASEDN:
-			      strncpy(f->cur_filter->basedn, f->cur_string, MAX_DN_LEN - 1);
+			      strncpy(f->cur_filter->basedn, f->cur_string,
+				      MAX_DN_LEN - 1);
 			      break;
 			 }
 		    }
@@ -501,8 +529,9 @@ void process_rcfile(char *rcfile)
 			      f->sp--;
 			 }
 			 else {
-			      sprintf(errstr, "line %d: can't close %s here\n",
-				      f->line, f->cur_string);
+			      snprintf(errstr, sizeof(errstr),
+				       _("line %d: can't close %s here\n"),
+				       f->line, f->cur_string);
 			      error_push(f->err_context, errstr);
 			 }
 		    }
@@ -543,11 +572,11 @@ char *filename_config(int context)
 
      home = homedir();
      if(home == NULL) {
-	  error_push(context, "you have no home directory!");
+	  error_push(context, _("you have no home directory!"));
 	  return(NULL);
      }
 
-     sprintf(rcpath, "%s/%s", home, RCFILE);
+     snprintf(rcpath, sizeof(rcpath), "%s/%s", home, RCFILE);
      free(home);
 
      return(rcpath);
@@ -561,7 +590,7 @@ void load_config(void)
      int load_context, rcsize;
      char *rcfile, *rcpath, errstr[MAX_ERRMSG_SIZE];
 
-     load_context = error_new_context("Error loading configfile");
+     load_context = error_new_context(_("Error loading configfile"));
 
      rcpath = filename_config(load_context);
      if(!rcpath) {
@@ -577,7 +606,12 @@ void load_config(void)
 
      /* refuse to read config file if world readable (bind passwords) */
      if( (sfile.st_mode & ~S_IFMT) != 00600 ) {
-	  sprintf(errstr, RCFILE_MODE_MSG, rcpath);
+	  snprintf(errstr, sizeof(errstr), 
+		   _("%s is group and/or world readable.\n"
+		     "This file can contain passwords in cleartext,\n"
+		     "and should be mode 0600.\n\n"
+		     "Continuing with default settings...\n"),
+		   rcpath);
 	  error_push(load_context, errstr);
 	  error_flush(load_context);
 	  return;
@@ -605,9 +639,9 @@ void config_write(struct writeconfig *wc, char *string)
      int i;
 
      for(i = 0; i < wc->indent; i++)
-	  fprintf(wc->outfile, CONFIG_INDENT_STRING);
+	  fprintf(wc->outfile, "%s", CONFIG_INDENT_STRING);
 
-     fprintf(wc->outfile, string);
+     fprintf(wc->outfile, "%s", string);
 
 }
 
@@ -616,7 +650,8 @@ void config_write_bool(struct writeconfig *wc, int value, char *entity)
 {
      char outstr[128];
 
-     sprintf(outstr, "<%s>%s</%s>\n", entity, value ? "True" : "False", entity);
+     snprintf(outstr, sizeof(outstr),
+	      "<%s>%s</%s>\n", entity, value ? "True" : "False", entity);
      config_write(wc, outstr);
 
 }
@@ -626,7 +661,8 @@ void config_write_int(struct writeconfig *wc, int value, char *entity)
 {
      char outstr[128];
 
-     sprintf(outstr, "<%s>%d</%s>\n", entity, value, entity);
+     snprintf(outstr, sizeof(outstr), 
+	      "<%s>%d</%s>\n", entity, value, entity);
      config_write(wc, outstr);
 
 }
@@ -637,12 +673,12 @@ void config_write_string(struct writeconfig *wc, char *value, char *entity)
      int i, p;
      char outstr[1024];
 
-     sprintf(outstr, "<%s>", entity);
+     snprintf(outstr, sizeof(outstr), "<%s>", entity);
 
      /* quick-and-dirty escape < and > */
      for(i = 0; value[i]; i++) {
 	  if(value[i] == '<' || value[i] == '>') {
-	       strcat(outstr, "\\");
+	       strcat(outstr, "\\"); /* FIXME: Buffer overflow probs  */
 	  }
 	  p = strlen(outstr);
 	  outstr[p++] = value[i];
@@ -653,7 +689,6 @@ void config_write_string(struct writeconfig *wc, char *value, char *entity)
      strcat(outstr, entity);
      strcat(outstr, ">\n");
      config_write(wc, outstr);
-
 }
 
 
@@ -676,7 +711,7 @@ void save_config(void)
      int write_context;
      char *rcpath, errstr[256];
 
-     write_context = error_new_context("Error writing configfile");
+     write_context = error_new_context(_("Error writing configfile"));
 
      server = config.ldapservers;
      wc = MALLOC(sizeof(struct writeconfig), "struct writeconfig");
@@ -690,8 +725,9 @@ void save_config(void)
 
      wc->outfile = fopen(rcpath, "w");
      if(!wc->outfile) {
-	  sprintf(errstr, "unable to open %s for writing:\n%s\n", rcpath,
-		  strerror(errno));
+	  snprintf(errstr, sizeof(errstr), 
+		   _("unable to open %s for writing:\n%s\n"), rcpath,
+		   strerror(errno));
 	  error_push(write_context, errstr);
 	  error_flush(write_context);
 	  return;
@@ -736,6 +772,10 @@ void save_config(void)
 	       config_write_bool(wc, server->cacheconn, "cache-connection");
 	  if(server->enabletls != DEFAULT_ENABLETLS)
 	       config_write_bool(wc, server->enabletls, "enable-tls");
+	  if(server->local_cache_timeout != DEFAULT_LOCAL_CACHE_TIMEOUT)
+	       config_write_int(wc, server->local_cache_timeout, "local-cache-timeout");
+	  if(server->ask_pw != DEFAULT_ASK_PW)
+	       config_write_bool(wc, server->ask_pw, "ask-pw");
 
 	  wc->indent--;
 	  config_write(wc, "</ldapserver>\n\n");
@@ -796,6 +836,9 @@ void save_config(void)
 
 void init_ldapserver(struct ldapserver *server)
 {
+     /* intitialize everything to nul-bytes first. This should help
+	when using strncpy to put stuff into the char arrays */
+     memset(server, 0, sizeof(struct ldapserver));
 
      server->name[0] = '\0';
      server->ldaphost[0] = '\0';
@@ -803,16 +846,21 @@ void init_ldapserver(struct ldapserver *server)
      server->basedn[0] = '\0';
      server->binddn[0] = '\0';
      server->bindpw[0] = '\0';
+     server->enteredpw[0] = '\0';
      server->bindtype = DEFAULT_BINDTYPE;
-     strncpy(server->searchattr, DEFAULT_SEARCHATTR, MAX_ATTR_LEN);
+     strncpy(server->searchattr, DEFAULT_SEARCHATTR, MAX_ATTR_LEN - 1);
      server->maxentries = DEFAULT_MAXENTRIES;
      server->cacheconn = DEFAULT_CACHECONN;
      server->enabletls = DEFAULT_ENABLETLS;
+     server->local_cache_timeout = DEFAULT_LOCAL_CACHE_TIMEOUT;
+     server->ask_pw = DEFAULT_ASK_PW;
      server->connection = NULL;
+     server->missing_closes = 0;
      server->ss = NULL;
      server->flags = 0;
      server->next = NULL;
-
+     server->version = LDAP_VERSION2;
+     server->server_down = 0;
 }
 
 
@@ -838,13 +886,19 @@ void init_config(void)
 	  /* no ldapserver defined in configfile */
 	  default_server = new_ldapserver();
 	  init_ldapserver(default_server);
-	  strcpy(default_server->name, "localhost");
-	  strcpy(default_server->ldaphost, "localhost");
-	  strcpy(default_server->searchattr, "cn");
+	  /* here we are happy with strcpy - what we copy is
+	     definitely short enough */
+	  strcpy(default_server->name, "localhost"); /* Flawfinder: ignore */
+	  strcpy(default_server->ldaphost, "localhost"); /* Flawfinder: ignore */
+	  strcpy(default_server->searchattr, "cn");/* Flawfinder: ignore */
 	  default_server->ldapport = 389;
 	  config.ldapservers = default_server;
      }
 
 }
 
-
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
