@@ -21,6 +21,10 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: configfile.h,v 1.14 2002/07/03 18:49:45 stamfest Exp $ */
+
+#ifndef GQ_CONFIGFILE_H_INCLUDED
+#define GQ_CONFIGFILE_H_INCLUDED
 
 #include <stdio.h>
 
@@ -31,11 +35,6 @@
 
 #define RCFILE           ".gq"
 
-#define RCFILE_MODE_MSG "%s is world readable.\nThis file can contain " \
-			"passwords,\n and should be mode 0600.\n\n" \
-			"Continuing with default settings...\n"
-
-
 /* tokens for config file */
 #define T_UNKNOWN             0
 #define T_XML_VERSION         1
@@ -65,6 +64,8 @@
 #define T_CACHECONN          30
 #define T_ENABLETLS          31
 #define T_LDIF_FORMAT        32
+#define T_LOCAL_CACHE_TIMEOUT 33
+#define T_ASK_PW	     34
 
 /* template tokens */
 #define T_TEMPLATE           40
@@ -85,6 +86,7 @@
 
 #define BINDTYPE_SIMPLE        0
 #define BINDTYPE_KERBEROS      1
+#define BINDTYPE_SASL          2
 
 #define MAX_ENTITY_LEN        64   /* not using XML attributes anyway */
 #define MAX_DATA_LEN         128
@@ -94,6 +96,9 @@
 #define DEFAULT_LDIFFORMAT   LDIF_UMICH
 #define DEFAULT_CACHECONN      1
 #define DEFAULT_ENABLETLS      0
+#define DEFAULT_LOCAL_CACHE_TIMEOUT -1
+#define DEFAULT_LOCAL_CACHE_SIZE (512*1024)
+#define DEFAULT_ASK_PW	       1
 
 enum searchtype {
      ST_SEARCH,
@@ -178,3 +183,13 @@ void config_write_string(struct writeconfig *wc, char *value, char *entity);
 void config_write_string_ne(struct writeconfig *wc, char *value, char *entity);
 void save_config(void);
 void init_config(void);
+
+extern struct gq_config config;
+
+#endif
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
