@@ -21,9 +21,13 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: debug.h,v 1.3 2002/06/05 15:49:49 stamfest Exp $ */
+
+#ifndef GQ_DEBUG_H_INCLUDED
+#define GQ_DEBUG_H_INCLUDED
 
 #include <stdlib.h>
-
+#include "config.h"
 
 #ifdef DEBUG
 
@@ -38,8 +42,27 @@
 #endif
 
 
+#ifdef MEMDEBUG
+#  ifdef HAVE_MALLINFO
+#    include <malloc.h>
+#    include <gtk/gtk.h>
+
+void init_memstat_timer();
+
+#  endif /* HAVE_MALLINFO */
+#endif /* MEMDEBUG */
+
 void *gq_malloc(size_t size, char *msg);
 void gq_free(void *mem, char *msg);
 void report_num_mallocs(void);
 void dump_ldapservers(void);
 
+
+#endif
+
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
