@@ -21,12 +21,33 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: errorchain.h,v 1.5 2002/06/27 08:21:20 stamfest Exp $ */
+
+#ifndef GQ_ERRORCHAIN_H_INCLUDED
+#define GQ_ERRORCHAIN_H_INCLUDED
+
+#include <ldap.h>		/* LDAP */
 
 int error_new_context(char *title);
-void error_push(int context, char *msg);
+void error_push_production(int context, char *msg);
+
+#ifdef ERROR_SHOW_LINE
+
+void error_push_debug(int context, char *msg, char *file, int line);
+#  define error_push(c, m) error_push_debug((c), (m), __FILE__, __LINE__)
+
+#else
+#  define error_push(c, m) error_push_production((c), (m))
+#endif
+
+
 struct errchain *error_chain_by_context(int q);
 void error_flush(int context);
 void error_popup(char *title, char *message);
+void error_clear(int context);
+void error_free(int context);
+
+void push_ldap_addl_error(LDAP *ld, int context);
 
 #define MAX_ERRTITLE_SIZE   128
 #define MAX_ERRMSG_SIZE     256
@@ -43,3 +64,10 @@ struct errchain {
      struct errchain *next;
 };
 
+#endif
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
