@@ -25,7 +25,9 @@
 #include <config.h>
 
 #include <string.h>
+#include <stdio.h>
 #if defined(HAVE_ISWSPACE)
+#include <wchar.h>
 #include <wctype.h>
 #else
 #include <ctype.h>
@@ -41,6 +43,7 @@
 #include "filter.h"
 #include "errorchain.h"
 #include "debug.h"
+#include "i18n.h"
 
 
 extern GtkWidget *mainwin;
@@ -102,7 +105,7 @@ void add_filter(GtkWidget *filternamebox)
 	  group = group->next;
      }
      if(searchtype != ST_FILTER) {
-	  single_warning_popup("You must select 'filter', not 'search'");
+	  single_warning_popup(_("You must select 'filter', not 'search'"));
 	  return;
      }
 
@@ -110,8 +113,10 @@ void add_filter(GtkWidget *filternamebox)
      searchstring = gtk_editable_get_chars(GTK_EDITABLE(focusbox), 0, -1);
 
      /* ignore if the filter is empty */
-     if(searchstring[0] == '\0')
+     if(searchstring[0] == '\0') {
+	  g_free(searchstring);
 	  return;
+     }
 
      if(filternamebox == NULL) {
 	  name_popup();
@@ -129,8 +134,10 @@ void add_filter(GtkWidget *filternamebox)
 	  filterlist = config.filters;
 	  while(filterlist) {
 	       filter = (struct gq_filter *) filterlist->data;
-	       if(!strncasecmp(filter->name, filtername, MAX_FILTERNAME_LEN - 1)) {
-		    sprintf(msg, "There is already a filter called '%s'", filtername);
+	       if(!strncasecmp(filter->name, filtername, 
+			       MAX_FILTERNAME_LEN - 1)) {
+		    snprintf(msg, sizeof(msg), 
+			     _("There is already a filter called '%s'"), filtername);
 		    single_warning_popup(msg);
 		    return;
 	       }
@@ -141,6 +148,7 @@ void add_filter(GtkWidget *filternamebox)
 	  filter = new_filter();
 	  strncpy(filter->name, filtername, MAX_FILTERNAME_LEN - 1);
 	  strncpy(filter->ldapfilter, searchstring, MAX_LDAPFILTER_LEN - 1);
+	  g_free(searchstring);
 
 	  /* check state of Remember... button, and get servername/basedn if active */
 	  button = gtk_object_get_data(GTK_OBJECT(filternamebox), "saveserver");
@@ -182,11 +190,11 @@ void add_filter(GtkWidget *filternamebox)
 
 void name_popup(void)
 {
-     GtkWidget *window, *vbox1, *vbox2, *label, *filternamebox, *button;
+     GtkWidget *window, *vbox1, *vbox2, *hbox0, *label, *filternamebox, *button;
 
      window = gtk_dialog_new();
      gtk_container_border_width(GTK_CONTAINER(window), 12);
-     gtk_window_set_title(GTK_WINDOW(window), "Filter name");
+     gtk_window_set_title(GTK_WINDOW(window), _("Filter name"));
      gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
      gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
                                (GtkSignalFunc) gtk_widget_destroy,
@@ -198,7 +206,7 @@ void name_popup(void)
      vbox1 = GTK_DIALOG(window)->vbox;
      gtk_widget_show(vbox1);
 
-     label = gtk_label_new("Filter name");
+     label = gtk_label_new(_("Filter name"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(vbox1), label, TRUE, TRUE, 0);
@@ -212,7 +220,7 @@ void name_popup(void)
      gtk_box_pack_start(GTK_BOX(vbox1), filternamebox, TRUE, TRUE, 0);
 
      /* Save server and base DN */
-     button = gtk_check_button_new_with_label("Remember server and base DN");
+     button = gtk_check_button_new_with_label(_("Remember server and base DN"));
      gtk_object_set_data(GTK_OBJECT(filternamebox), "saveserver", button);
      GTK_WIDGET_UNSET_FLAGS(GTK_CHECK_BUTTON(button), GTK_CAN_FOCUS);
      gtk_widget_show(button);
@@ -221,10 +229,14 @@ void name_popup(void)
      vbox2 = GTK_DIALOG(window)->action_area;
      gtk_widget_show(vbox2);
 
-     button = gtk_button_new_with_label("   OK   ");
+     hbox0 = gtk_hbutton_box_new();
+     gtk_widget_show(hbox0);
+     gtk_box_pack_start(GTK_BOX(vbox2), hbox0, TRUE, TRUE, 0);
+
+     button = gtk_button_new_with_label(_("OK"));
      gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                                GTK_SIGNAL_FUNC(add_filter), GTK_OBJECT(filternamebox));
-     gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, FALSE, 0);
+     gtk_box_pack_start(GTK_BOX(hbox0), button, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
      GTK_WIDGET_SET_FLAGS(button, GTK_RECEIVES_DEFAULT);
      gtk_widget_grab_default(button);
@@ -293,22 +305,25 @@ void filter_selected(struct gq_filter *filter)
 	  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(searchbase_combo)->entry), filter->basedn);
      }
 
+     gtk_signal_emit_by_name(GTK_OBJECT(focusbox), "activate");
+
 }
 
 
 void show_filters(void)
 {
-     GtkWidget *window, *vbox1, *hbox1, *hbox2, *new_button, *delete_button;
+     GtkWidget *window, *vbox1, *hbox0, *hbox1, *hbox2, *new_button, *delete_button;
      GtkWidget *copy_button, *ok_button, *scrwin, *filter_clist;
      GList *filterlist;
      struct gq_filter *filter;
      int i, row;
-     char *titles[] = { "Filter name", "Server", "Base DN", "Filter" };
+     char *titles[] = { _("Filter name"), _("Server"),
+			_("Base DN"), _("Filter") };
      char *clist_content[5];
 
      window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_container_border_width(GTK_CONTAINER(window), 12);
-     gtk_window_set_title(GTK_WINDOW(window), "Filters");
+     gtk_window_set_title(GTK_WINDOW(window), _("Filters"));
      gtk_window_set_default_size(GTK_WINDOW(window), 670, 350);
      gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
                                (GtkSignalFunc) gtk_widget_destroy,
@@ -321,23 +336,27 @@ void show_filters(void)
      gtk_widget_show(vbox1);
      gtk_container_add(GTK_CONTAINER(window), vbox1);
 
-     hbox1 = gtk_hbox_new(FALSE, 0);
-     gtk_widget_show(hbox1);
-     gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
+     hbox0 = gtk_hbox_new(FALSE, 0);
+     gtk_widget_show(hbox0);
+     gtk_box_pack_start(GTK_BOX(vbox1), hbox0, FALSE, FALSE, 0);
 
-     new_button = gtk_button_new_with_label("  New  ");
+     hbox1 = gtk_hbutton_box_new();
+     gtk_widget_show(hbox1);
+     gtk_box_pack_start(GTK_BOX(hbox0), hbox1, FALSE, FALSE, 0);
+
+     new_button = gtk_button_new_with_label(_("New"));
      GTK_WIDGET_UNSET_FLAGS(new_button, GTK_CAN_DEFAULT);
      GTK_WIDGET_UNSET_FLAGS(new_button, GTK_CAN_FOCUS);
      gtk_widget_show(new_button);
      gtk_box_pack_start(GTK_BOX(hbox1), new_button, FALSE, FALSE, 0);
 
-     delete_button = gtk_button_new_with_label("  Delete  ");
+     delete_button = gtk_button_new_with_label(_("Delete"));
      GTK_WIDGET_UNSET_FLAGS(delete_button, GTK_CAN_DEFAULT);
      GTK_WIDGET_UNSET_FLAGS(delete_button, GTK_CAN_FOCUS);
      gtk_widget_show(delete_button);
      gtk_box_pack_start(GTK_BOX(hbox1), delete_button, FALSE, FALSE, 5);
 
-     copy_button = gtk_button_new_with_label("  Copy  ");
+     copy_button = gtk_button_new_with_label(_("Copy"));
      GTK_WIDGET_UNSET_FLAGS(copy_button, GTK_CAN_DEFAULT);
      GTK_WIDGET_UNSET_FLAGS(copy_button, GTK_CAN_FOCUS);
      gtk_widget_show(copy_button);
@@ -378,13 +397,13 @@ void show_filters(void)
      }
      gtk_clist_thaw(GTK_CLIST(filter_clist));
 
-     hbox2 = gtk_hbox_new(FALSE, 20);
+     hbox2 = gtk_hbutton_box_new(); /* (FALSE, 20); */
      gtk_widget_show(hbox2);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox2, FALSE, FALSE, 5);
 
-     ok_button = gtk_button_new_with_label("   OK   ");
+     ok_button = gtk_button_new_with_label(_("OK"));
      gtk_widget_show(ok_button);
-     gtk_box_pack_start(GTK_BOX(hbox2), ok_button, FALSE, FALSE, 5);
+     gtk_box_pack_start(GTK_BOX(hbox2), ok_button, FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
      GTK_WIDGET_SET_FLAGS(ok_button, GTK_RECEIVES_DEFAULT);
      gtk_widget_grab_default(ok_button);
@@ -528,7 +547,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
      gtk_object_set_data(GTK_OBJECT(window), "filter_row", GINT_TO_POINTER(row));
 
      gtk_container_border_width(GTK_CONTAINER(window), 12);
-     gtk_window_set_title(GTK_WINDOW(window), "Edit filter");
+     gtk_window_set_title(GTK_WINDOW(window), _("Edit filter"));
      gtk_signal_connect_object(GTK_OBJECT(window), "destroy",
                                (GtkSignalFunc) gtk_widget_destroy,
                                (gpointer) window);
@@ -548,7 +567,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
      gtk_table_set_col_spacings(GTK_TABLE(table1), 13);
 
      /* Filter name */
-     label = gtk_label_new("Filter name");
+     label = gtk_label_new(_("Filter name"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -563,7 +582,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
 		      GTK_EXPAND | GTK_FILL, 0, 0, 0);
 
      /* Server */
-     label = gtk_label_new("Server");
+     label = gtk_label_new(_("Server"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 1, 2,
@@ -578,7 +597,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
      /* Base DN */
-     label = gtk_label_new("Base DN");
+     label = gtk_label_new(_("Base DN"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
@@ -596,7 +615,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
      gtk_widget_show(hbox1);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 0);
 
-     indent = gtk_toggle_button_new_with_label("  Indent  ");
+     indent = gtk_toggle_button_new_with_label(_("  Indent  "));
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(indent), TRUE);
      GTK_WIDGET_UNSET_FLAGS(indent, GTK_CAN_FOCUS);
      gtk_widget_show(indent);
@@ -628,11 +647,11 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
 
      gtk_widget_show(editbox);
 
-     hbox2 = gtk_hbox_new(FALSE, 0);
+     hbox2 = gtk_hbutton_box_new(); /* box_new(FALSE, 0); */
      gtk_widget_show(hbox2);
-     gtk_box_pack_start(GTK_BOX(vbox1), hbox2, FALSE, FALSE, 10);
+     gtk_box_pack_start(GTK_BOX(vbox1), hbox2, TRUE, TRUE, 10);
 
-     save = gtk_button_new_with_label("   Save   ");
+     save = gtk_button_new_with_label(_("Save"));
      gtk_signal_connect_object(GTK_OBJECT(save), "clicked",
                                GTK_SIGNAL_FUNC(save_filter), GTK_OBJECT(window));
      gtk_box_pack_start(GTK_BOX(hbox2), save, FALSE, FALSE, 0);
@@ -641,7 +660,7 @@ void edit_filter(GtkCList *filter_clist, int is_new_filter, int row, struct gq_f
      gtk_widget_grab_default(save);
      gtk_widget_show(save);
 
-     cancel = gtk_button_new_with_label("  Cancel  ");
+     cancel = gtk_button_new_with_label(_("Cancel"));
      GTK_WIDGET_UNSET_FLAGS(cancel, GTK_CAN_FOCUS);
      gtk_box_pack_end(GTK_BOX(hbox2), cancel, FALSE, FALSE, 0);
      gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked",
@@ -704,14 +723,14 @@ void save_filter(GtkWidget *window)
      row = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(window), "filter_row"));
 
      if(!filtername || !filtername[0]) {
-	  single_warning_popup("You must fill in a name for the filter");
+	  single_warning_popup(_("You must fill in a name for the filter"));
 	  g_free(filtertext);
 	  return;
      }
 
      if(!filter || !servername || !basedn ) {
 	  /* shouldn't happen */
-	  error_popup("Save failed", "Unknown error");
+	  error_popup(_("Save failed"), _("Unknown error"));
 	  g_free(filtertext);
 	  return;
      }
@@ -721,7 +740,9 @@ void save_filter(GtkWidget *window)
 
      if(is_a_new_filter) {
 	  if(check_filtername(filtername)) {
-	       sprintf(msg, "There is already a filter called '%s'", filtername);
+	       snprintf(msg, sizeof(msg), 
+			_("There is already a filter called '%s'"),
+			filtername);
 	       single_warning_popup(msg);
 	       g_free(flattened);
 	       FREE(filter, "struct gq_filter");
@@ -767,7 +788,9 @@ void save_filter(GtkWidget *window)
 	  if(strncasecmp(filter->name, filtername, MAX_FILTERNAME_LEN)) {
 	       /* existing filter was renamed */
 	       if(check_filtername(filtername)) {
-		    sprintf(msg, "There is already a filter called '%s'", filtername);
+		    snprintf(msg, sizeof(msg), 
+			     _("There is already a filter called '%s'"),
+			     filtername);
 		    single_warning_popup(msg);
 		    g_free(flattened);
 		    return;
@@ -919,7 +942,7 @@ char *unindent_filter(char *indented)
 	  default:
 	       /* caught spaces before -- this is testing for \n etc */
 #if defined(HAVE_ISWSPACE)
-	       if(!iswspace(c))
+	       if(!iswspace(btowc(c)))
 #else
                if(!isspace(c))
 #endif /* HAVE_ISWSPACE */
@@ -967,3 +990,9 @@ void indent_toggled(GtkToggleButton *indent, gpointer editbox)
      }
 
 }
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+*/
