@@ -21,8 +21,11 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: formfill.c,v 1.27 2002/06/18 22:07:14 stamfest Exp $ */
+
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <glib.h>
 
@@ -32,52 +35,66 @@
 #include "common.h"
 #include "util.h"
 #include "formfill.h"
-
-extern struct tokenlist cryptmap[];
-
+#include "ldif.h"
+#include "syntax.h"
+#include "schema.h"
+#include "encode.h"
+#include "i18n.h"
 
 struct formfill *new_formfill(void)
 {
      struct formfill *form;
 
-     if( (form = (struct formfill *) g_malloc(sizeof(struct formfill))) == NULL)
+     if( (form = (struct formfill *) g_new0(struct formfill, 1)) == NULL)
 	  return(NULL);
+
      form->attrname[0] = '\0';
      form->num_inputfields = 1;
      form->displaytype = DISPLAYTYPE_ENTRY;
+     form->dt_handler = get_dt_handler(form->displaytype);
      form->flags = 0;
      form->values = NULL;
 
+     form->label = form->vbox = form->morebutton = NULL;
+     form->widgetList = NULL;
+
      return(form);
 }
 
 
 void free_formlist(GList *formlist)
 {
-
-     while(formlist) {
-	  free_formfill( (struct formfill *) formlist->data);
-	  formlist = formlist->next;
+     GList *n, *f;
+     
+     for (f = formlist ; f ; f = n) {
+	  n = f->next;
+	  free_formfill( (struct formfill *) f->data);
      }
      g_list_free(formlist);
-
 }
 
 
+void free_formfill_values(struct formfill *form) {
+     if (form) {
+	  GList *vals = form->values;
+	  
+	  if (vals) {
+	       while(vals) {
+		    g_byte_array_free((GByteArray*)vals->data, TRUE);
+		    vals->data = NULL;
+		    vals = vals->next;
+	       }
+	       g_list_free(form->values);
+	       form->values = NULL;
+	  }
+     }
+}
+
 void free_formfill(struct formfill *form)
 {
-     GList *vals;
-
-     if((vals = form->values) != NULL) {
-	  while(vals) {
-	       g_free(vals->data);
-	       vals = vals->next;
-	  }
-	  g_list_free(form->values);
-     }
-
+     free_formfill_values(form);
+     if (form->widgetList) g_list_free(form->widgetList);
      g_free(form);
-
 }
 
 
@@ -108,6 +125,16 @@ GList *formlist_append(GList *formlist, struct formfill *form)
      return(formlist);
 }
 
+char *attr_strip(const char *attr)
+{
+     char *c = g_strdup(attr);
+     char *d = strchr(c, ';');
+
+     if (d) {
+	  *d = 0;
+     }
+     return c;
+}
 
 GList *formlist_from_entry(struct ldapserver *server, char *dn, int ocvalues_only)
 {
@@ -118,14 +145,18 @@ GList *formlist_from_entry(struct ldapserver *server, char *dn, int ocvalues_onl
      BerElement *ber;
      struct formfill *form;
      int msg, i;
-     char *attr, **vals;
+     char *attr; /* , **vals; */
+     struct berval **bervals;
+     char *c = NULL;
 
      formlist = NULL;
 
      set_busycursor();
 
      message = g_string_sized_new(128);
-     g_string_sprintf(message, "fetching %s from %s", dn, server->name);
+     g_string_sprintf(message, _("fetching %s from %s"), 
+		      c = decoded_string(dn), server->name);
+     if (c) free(c);
      statusbar_msg(message->str);
      g_string_free(message, TRUE);
 
@@ -134,38 +165,63 @@ GList *formlist_from_entry(struct ldapserver *server, char *dn, int ocvalues_onl
           return(NULL);
      }
 
-     msg = ldap_search_s(ld, dn, LDAP_SCOPE_BASE, "(objectclass=*)", NULL, 0, &res);
+     msg = ldap_search_s(ld, dn, LDAP_SCOPE_BASE, "(objectclass=*)", 
+			 NULL, 0, &res);
+
+     if (msg == LDAP_SERVER_DOWN) {
+	  server->server_down++;
+     }
 
      if(msg != LDAP_SUCCESS) {
 	  statusbar_msg(ldap_err2string(msg));
 	  set_normalcursor();
+	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
      if (res) {
 	  entry = ldap_first_entry(ld, res);
 	  if(entry) {
-	       for(attr = ldap_first_attribute(ld, entry, &ber); attr; attr = ldap_next_attribute(ld, entry, ber)) {
-		    if( (form = new_formfill()) == NULL)
+	       char *c;
+	       for(attr = ldap_first_attribute(ld, entry, &ber); attr ;
+		   attr = ldap_next_attribute(ld, entry, ber)) {
+		    if( (form = new_formfill()) == NULL) {
+			 ldap_memfree(attr);
 			 break;
-		    strncpy(form->attrname, attr, MAX_ATTR_LEN);
-		    if(ocvalues_only == 0 || !strcasecmp(attr, "objectClass")) {
-			 vals = ldap_get_values(ld, res, attr);
-			 if(vals) {
-			      for(i = 0; vals[i]; i++)
-				   form->values = g_list_append(form->values, g_strdup(vals[i]));
+		    }
+		    c = attr_strip(attr);
+		    strncpy(form->attrname, c, MAX_ATTR_LEN);
+		    if (c) g_free(c);
+
+		    if(ocvalues_only == 0 ||
+		       !strcasecmp(attr, "objectClass")) {
+			 bervals = ldap_get_values_len(ld, res, attr);
+			 if(bervals) {
+			      for(i = 0; bervals[i]; i++) {
+				   GByteArray *gb = g_byte_array_new();
+				   g_byte_array_append(gb, 
+						       bervals[i]->bv_val,
+						       bervals[i]->bv_len);
+				   form->values = g_list_append(form->values,
+								gb);
+			      }
+
 			      form->num_inputfields = i;
-			      ldap_value_free(vals);
+			      ldap_value_free_len(bervals);
 			 }
 		    }
-		    set_displaytype(form);
+
+		    set_displaytype(server, form);
 		    formlist = g_list_append(formlist, form);
+		    ldap_memfree(attr);
 	       }
+#ifndef HAVE_OPENLDAP12
 	       /* this bombs out with openldap 1.2.x libs... */
-	       /*
-		 if(ber)
-		      ber_free(ber, 0);
-	       */
+	       /* PSt: is this still true??? - introduced a configure check */
+	       if(ber) {
+		    ber_free(ber, 0);
+	       }
+#endif
 	  }
 	  ldap_msgfree(res);
      }
@@ -188,15 +244,24 @@ GList *dup_formlist(GList *oldlist)
 	  oldform = (struct formfill *) oldlist->data;
 	  if( (newform = new_formfill()) == NULL)
 	       break;
-	  strcpy(newform->attrname, oldform->attrname);
+	  /* no problem with NUL bytes here - new_formfill returns a
+             completly NUL inited object */
+	  strncpy(newform->attrname, oldform->attrname,
+		  sizeof(newform->attrname));
 	  newform->num_inputfields = oldform->num_inputfields;
 	  newform->displaytype = oldform->displaytype;
+	  newform->dt_handler = oldform->dt_handler;
 	  newform->flags = oldform->flags;
 	  oldval = oldform->values;
 	  newval = NULL;
 	  while(oldval) {
-	       if(oldval->data)
-		    newval = g_list_append(newval, g_strdup((char *) (oldval->data)));
+	       if(oldval->data) {
+		    GByteArray *old = (GByteArray*) (oldval->data);
+		    GByteArray *gb = g_byte_array_new();
+		    g_byte_array_append(gb, old->data, old->len);
+
+		    newval = g_list_append(newval, gb);
+	       }
 
 	       oldval = oldval->next;
 	  }
@@ -239,8 +304,15 @@ void dump_formlist(GList *formlist)
 	  values = form->values;
 
 	  while(values) {
-	       if(values->data)
-		    printf("\t%s\n", (char *) values->data);
+	       if(values->data) {
+		    GByteArray *d = (GByteArray*) values->data;
+/*  		    GByteArray *gb = form->dt_handler->encode(d->data, d->len); */
+		    printf("\t");
+		    fwrite(d->data, d->len, 1, stdout);
+		    printf("\n");
+
+/*  		    g_byte_array_free(gb, TRUE); */
+	       }
 	       values = values->next;
 	  }
 
@@ -265,39 +337,69 @@ struct formfill *lookup_attribute(GList *formlist, char *attr)
      return(NULL);
 }
 
+/* Returns an existing formfill object (if any) for attribute attr
+   from the passed-in list.  The server_schema gets consulted to also
+   check for aliases. The canonical attribute type may be retrieved
+   when passing a non NULL value for attrtype */
+struct formfill *lookup_attribute_using_schema(GList *formlist, char *attr,
+					       struct server_schema *schema,
+					       LDAPAttributeType **attrtype)
+{
+     struct formfill *form;
+
+#ifdef HAVE_LDAP_STR2OBJECTCLASS
+     char **n;
+     LDAPAttributeType *trythese = NULL;
+
+     if (attrtype) *attrtype = NULL;
+
+     if (schema) {
+	  trythese = find_canonical_at_by_at(schema, attr);
+	  if (attrtype) *attrtype = trythese;
+     }
+#endif /* HAVE_LDAP_STR2OBJECTCLASS */
+
+     for( ; formlist ; formlist = formlist->next ) {
+	  form = (struct formfill *) formlist->data;
+
+#ifdef HAVE_LDAP_STR2OBJECTCLASS
+	  if (trythese && form->attrname) {
+	       for (n = trythese->at_names ; n && *n ; n++) {
+		    if(!strcasecmp(form->attrname, *n)) return(form);
+	       }
+	  } else {
+	       if(form->attrname && !strcasecmp(form->attrname, attr))
+		    return(form);
+	  }
+#else /* HAVE_LDAP_STR2OBJECTCLASS */
+	  if(form->attrname && !strcasecmp(form->attrname, attr))
+	       return(form);
+#endif /* HAVE_LDAP_STR2OBJECTCLASS */
+     }
+
+     return(NULL);
+}
 
 /*
- * sure would be nice to use serverschema for this
+ * sure it is nice to use serverschema for this
+ *
+ */
+int find_displaytype(struct ldapserver *server, struct formfill *form)
+{
+     return get_display_type_of_attr(server, form->attrname);
+}
+
+/*
+ * now we use the serverschema for this
  *
  * hint, hint
  *
  */
-void set_displaytype(struct formfill *form)
+void set_displaytype(struct ldapserver *server, struct formfill *form)
 {
-     int i;
-     char *password, crypt_prefix[12];
-
-     if(!strcasecmp(form->attrname, "userPassword")) {
-	  form->displaytype = DISPLAYTYPE_PASSWORD;
-
-	  /* see if we can find a {crypt prefix} in the password, and set flags accordingly */
-	  form->flags &= !ENCODING_MASK;
-	  if(form->values) {
-	       password = form->values->data;
-	       if(password && password[0] == '{') {
-		    for(i = 1; password[i] && password[i] != '}' && i < 10; i++)
-			 crypt_prefix[i - 1] = password[i];
-		    crypt_prefix[i - 1] = '\0';
-
-	       }
-	       form->flags |= tokenize(cryptmap, crypt_prefix);
-	  }
-
-     }
-     else {
-	  form->displaytype = DISPLAYTYPE_ENTRY;
-     }
-
+     form->syntax = get_syntax_handler_of_attr(server, form->attrname, NULL);
+     form->displaytype = find_displaytype(server, form);
+     form->dt_handler = get_dt_handler(form->displaytype);
 }
 
 
@@ -326,3 +428,9 @@ void add_field_to_formfill(GList *formlist, char *attr)
 }
 
 
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
