@@ -21,8 +21,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: gq.h,v 1.2 2002/06/13 10:02:51 stamfest Exp $ */
+
+#ifndef GQ_GQ_H_INCLUDED
+#define GQ_GQ_H_INCLUDED
 
 #define FONT_BIG_FIXED     "-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*"
 
 
 void setup_fonts(void);
+
+#endif
