@@ -21,8 +21,14 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#ifndef GQ_LDIF_H_INCLUDED
+#define GQ_LDIF_H_INCLUDED
 
-void prepend_ldif_header(GString *out, struct ldapserver *server, GSList *bases);
+void prepend_ldif_header(GString *out, struct ldapserver *server,
+			 GList *bases);
 gboolean ldif_entry_out(GString *out, LDAP *ld, LDAPMessage *msg);
 gboolean ldif_line_out(GString *out, char *attr, char *value, int vlen);
 void b64_encode(GString *out, char *value, int vlen);
+void b64_decode(GByteArray *out, const char *value, int vlen);
+
+#endif
