@@ -40,6 +40,7 @@
 #include "template.h"
 #include "filter.h"
 #include "COPYING.h"
+#include "i18n.h"
 #include "debug.h"
 
 #include "../icons/logo.xpm"
@@ -53,9 +54,6 @@ GtkWidget *mainbook = NULL;
 GtkWidget *statusbar;
 GList *tablist = NULL;
 
-extern struct gq_config config;
-
-
 void fill_serverlist_combo(GtkWidget *combo)
 {
      GList *serverlist;
@@ -141,38 +139,49 @@ void cleanup(void)
 
 void create_mainwin(void)
 {
-     GtkWidget *main_vbox, *menubar, *menuitem, *submenu;
+     GtkWidget *outer_vbox, *main_vbox, *menubar, *menuitem, *submenu;
      GtkWidget *File, *menuFile, *New, *Close, *Quit;
      GtkWidget *Search, *Browse, *Schema;
      GtkWidget *menuHelp, *Help, *License, *About;
      GtkWidget *Filters, *menuFilters;
+     GtkWidget *handlebox;
      GtkAccelGroup *accel_group;
      GList *filterlist;
      struct gq_filter *filter;
 
      mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
      gtk_window_set_default_size(GTK_WINDOW(mainwin), 770, 478);
-     gtk_container_border_width(GTK_CONTAINER(mainwin), 4);
+     gtk_container_border_width(GTK_CONTAINER(mainwin), 0);
      gtk_signal_connect(GTK_OBJECT(mainwin), "destroy",
 			GTK_SIGNAL_FUNC(gq_exit),
 			NULL);
-     gtk_window_set_title(GTK_WINDOW(mainwin), "GQ");
-     gtk_window_set_policy(GTK_WINDOW(mainwin), TRUE, TRUE, FALSE);
+     gtk_window_set_title(GTK_WINDOW(mainwin), _("GQ"));
+     gtk_window_set_policy(GTK_WINDOW(mainwin), FALSE, TRUE, FALSE);
+
+     outer_vbox = gtk_vbox_new(FALSE, 2);
+     gtk_container_border_width(GTK_CONTAINER(outer_vbox), 0);
+     gtk_widget_show(outer_vbox);
+     gtk_container_add(GTK_CONTAINER(mainwin), outer_vbox);
 
      main_vbox = gtk_vbox_new(FALSE, 2);
      gtk_object_set_data(GTK_OBJECT(mainwin), "main_vbox", main_vbox);
+     gtk_container_border_width(GTK_CONTAINER(main_vbox), 4);
      gtk_widget_show(main_vbox);
-     gtk_container_add(GTK_CONTAINER(mainwin), main_vbox);
+     gtk_box_pack_end(GTK_BOX(outer_vbox), main_vbox, TRUE, TRUE, 1);
 
      accel_group = gtk_accel_group_new();
      gtk_accel_group_attach(accel_group, GTK_OBJECT(mainwin));
 
+     handlebox = gtk_handle_box_new();
+     gtk_widget_show(handlebox);
+     gtk_box_pack_start(GTK_BOX(outer_vbox), handlebox, FALSE, TRUE, 0);
+
      menubar = gtk_menu_bar_new();
      gtk_widget_show(menubar);
-     gtk_box_pack_start(GTK_BOX(main_vbox), menubar, FALSE, TRUE, 1);
+     gtk_container_add(GTK_CONTAINER(handlebox), menubar);
 
      /* File menu */
-     File = gtk_menu_item_new_with_label("File");
+     File = gtk_menu_item_new_with_label(_("File"));
      gtk_widget_show(File);
      gtk_container_add(GTK_CONTAINER(menubar), File);
 
@@ -180,14 +189,14 @@ void create_mainwin(void)
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(File), menuFile);
 
      /* File | New */
-     New = gtk_menu_item_new_with_label("New tab");
+     New = gtk_menu_item_new_with_label(_("New tab"));
      gtk_widget_show(New);
      gtk_container_add(GTK_CONTAINER(menuFile), New);
      submenu = gtk_menu_new();
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(New), submenu);
 
      /* File | New | Search */
-     Search = gtk_menu_item_new_with_label("Search");
+     Search = gtk_menu_item_new_with_label(_("Search"));
      gtk_widget_show(Search);
      gtk_menu_append(GTK_MENU(submenu), Search);
      gtk_signal_connect_object(GTK_OBJECT(Search), "activate",
@@ -197,7 +206,7 @@ void create_mainwin(void)
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
      /* File | New | Browse */
-     Browse = gtk_menu_item_new_with_label("Browse");
+     Browse = gtk_menu_item_new_with_label(_("Browse"));
      gtk_widget_show(Browse);
      gtk_menu_append(GTK_MENU(submenu), Browse);
      gtk_signal_connect_object(GTK_OBJECT(Browse), "activate",
@@ -212,7 +221,7 @@ void create_mainwin(void)
 			(gpointer) Browse);
 
      /* File | New | Schema */
-     Schema = gtk_menu_item_new_with_label("Schema");
+     Schema = gtk_menu_item_new_with_label(_("Schema"));
      gtk_widget_show(Schema);
      gtk_menu_append(GTK_MENU(submenu), Schema);
      gtk_widget_add_accelerator(Schema, "activate", accel_group, 'Z',
@@ -226,7 +235,7 @@ void create_mainwin(void)
 #endif
 
      /* File | Preferences */
-     menuitem = gtk_menu_item_new_with_label("Preferences");
+     menuitem = gtk_menu_item_new_with_label(_("Preferences"));
      gtk_widget_show(menuitem);
      gtk_container_add(GTK_CONTAINER(menuFile), menuitem);
      gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
@@ -236,7 +245,7 @@ void create_mainwin(void)
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
      /* File | Close */
-     Close = gtk_menu_item_new_with_label("Close");
+     Close = gtk_menu_item_new_with_label(_("Close"));
      gtk_widget_show(Close);
      gtk_container_add(GTK_CONTAINER(menuFile), Close);
      gtk_signal_connect_object(GTK_OBJECT(Close), "activate",
@@ -245,7 +254,7 @@ void create_mainwin(void)
 				GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
 
      /* File | Quit */
-     Quit = gtk_menu_item_new_with_label("Quit");
+     Quit = gtk_menu_item_new_with_label(_("Quit"));
      gtk_widget_show(Quit);
      gtk_container_add(GTK_CONTAINER(menuFile), Quit);
      gtk_signal_connect_object(GTK_OBJECT(Quit), "activate",
@@ -254,7 +263,7 @@ void create_mainwin(void)
 
 
      /* Filters menu */
-     Filters = gtk_menu_item_new_with_label("Filters");
+     Filters = gtk_menu_item_new_with_label(_("Filters"));
      gtk_widget_show(Filters);
      gtk_container_add(GTK_CONTAINER(menubar), Filters);
 
@@ -263,14 +272,14 @@ void create_mainwin(void)
      gtk_object_set_data(GTK_OBJECT(mainwin), "filtermenu", menuFilters);
 
      /* Filters | New */
-     New = gtk_menu_item_new_with_label("New filter");
+     New = gtk_menu_item_new_with_label(_("New filter"));
      gtk_widget_show(New);
      gtk_container_add(GTK_CONTAINER(menuFilters), New);
      submenu = gtk_menu_new();
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(New), submenu);
 
      /* Filters | New | From Search tab */
-     menuitem = gtk_menu_item_new_with_label("From Search tab");
+     menuitem = gtk_menu_item_new_with_label(_("From Search tab"));
      gtk_widget_show(menuitem);
      gtk_menu_append(GTK_MENU(submenu), menuitem);
      gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
@@ -278,7 +287,7 @@ void create_mainwin(void)
 			       NULL);
 
      /* Filters | New | Filter Editor */
-     menuitem = gtk_menu_item_new_with_label("Filter editor");
+     menuitem = gtk_menu_item_new_with_label(_("Filter editor"));
      gtk_widget_show(menuitem);
      gtk_menu_append(GTK_MENU(submenu), menuitem);
      gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
@@ -286,7 +295,7 @@ void create_mainwin(void)
 			       NULL);
 
      /* Filters | Edit Filters */
-     menuitem = gtk_menu_item_new_with_label("Edit Filters");
+     menuitem = gtk_menu_item_new_with_label(_("Edit Filters"));
      gtk_widget_show(menuitem);
      gtk_container_add(GTK_CONTAINER(menuFilters), menuitem);
      gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
@@ -314,7 +323,7 @@ void create_mainwin(void)
 
 
      /* Help menu */
-     Help = gtk_menu_item_new_with_label("Help");
+     Help = gtk_menu_item_new_with_label(_("Help"));
      gtk_widget_show(Help);
      gtk_container_add(GTK_CONTAINER(menubar), Help);
      gtk_menu_item_right_justify(GTK_MENU_ITEM(Help));
@@ -323,14 +332,14 @@ void create_mainwin(void)
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(Help), menuHelp);
 
      /* Help | License */
-     License = gtk_menu_item_new_with_label("License");
+     License = gtk_menu_item_new_with_label(_("License"));
      gtk_widget_show(License);
      gtk_container_add(GTK_CONTAINER(menuHelp), License);
      gtk_signal_connect(GTK_OBJECT(License), "activate",
 			GTK_SIGNAL_FUNC(create_license_window), NULL);
 
      /* Help | About */
-     About = gtk_menu_item_new_with_label("About");
+     About = gtk_menu_item_new_with_label(_("About"));
      gtk_widget_show(About);
      gtk_container_add(GTK_CONTAINER(menuHelp), About);
      gtk_signal_connect(GTK_OBJECT(About), "activate",
@@ -374,16 +383,16 @@ void new_modetab(int mode)
 
      switch(mode) {
      case SEARCH_MODE:
-	  label = gtk_label_new("Search");
+	  label = gtk_label_new(_("Search"));
 	  new_searchmode(hash);
 	  break;
      case BROWSE_MODE:
-	  label = gtk_label_new("Browse");
+	  label = gtk_label_new(_("Browse"));
 	  new_browsemode(hash);
 	  break;
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
      case SCHEMA_MODE:
-	  label = gtk_label_new("Schema");
+	  label = gtk_label_new(_("Schema"));
 	  new_schemamode(hash);
 	  break;
 #endif
@@ -521,7 +530,7 @@ void update_serverlist(void)
 
 void create_license_window(void)
 {
-     GtkWidget *window, *vbox1, *hbox, *vbox2, *text;
+     GtkWidget *window, *vbox1, *hbox, *vbox2, *text, *hbox0;
      GtkWidget *vscroll, *ok_button;
      GtkStyle *license_style;
 
@@ -529,7 +538,7 @@ void create_license_window(void)
      gtk_object_set_data(GTK_OBJECT(window), "close_func",
 			 close_license_window);
      gtk_widget_set_usize(window, 540, 400);
-     gtk_window_set_title(GTK_WINDOW(window), "License");
+     gtk_window_set_title(GTK_WINDOW(window), _("License"));
 
      vbox1 = GTK_DIALOG(window)->vbox;
      gtk_container_border_width(GTK_CONTAINER(vbox1), 20);
@@ -558,9 +567,14 @@ void create_license_window(void)
 
      vbox2 = GTK_DIALOG(window)->action_area;
      gtk_widget_show(vbox2);
-     ok_button = gtk_button_new_with_label("    OK    ");
+
+     hbox0 = gtk_hbutton_box_new();
+     gtk_widget_show(hbox0);
+     gtk_box_pack_start(GTK_BOX(vbox2), hbox0, TRUE, FALSE, 0);
+
+     ok_button = gtk_button_new_with_label(_("OK"));
      gtk_widget_show(ok_button);
-     gtk_box_pack_start(GTK_BOX(vbox2), ok_button,
+     gtk_box_pack_start(GTK_BOX(hbox0), ok_button,
 			FALSE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default(ok_button);
@@ -593,19 +607,19 @@ void close_license_window(GtkWidget *window)
 GtkWidget *create_about_window(void)
 {
      GtkWidget *about_window;
-     GtkWidget *dialog_vbox2, *hbox;
+     GtkWidget *dialog_vbox2, *hbox0, *hbox;
      GtkWidget *about_label;
      GtkWidget *dialog_action_area2;
      GtkWidget *about_ok_button;
      GtkWidget *pixmap;
      GdkPixmap *gq_icon;
      GdkBitmap *gq_icon_mask;
-     char title[32], about_text[256];
+     char title[32], about_text[512];
 
      about_window = gtk_dialog_new();
      gtk_widget_realize(GTK_WIDGET(about_window));
      gtk_container_border_width(GTK_CONTAINER(about_window), 12);
-     sprintf(title, "About GQ");
+     snprintf(title, sizeof(title), _("About GQ"));
      gtk_window_set_title(GTK_WINDOW(about_window), title);
      gtk_window_set_policy(GTK_WINDOW(about_window), FALSE, FALSE, FALSE);
 
@@ -624,7 +638,8 @@ GtkWidget *create_about_window(void)
      gtk_widget_show(pixmap);
      gtk_box_pack_start(GTK_BOX(hbox), pixmap, TRUE, TRUE, 10);
 
-     sprintf(about_text, "GQ %s\n\n\n%s", VERSION, about_blurb);
+     snprintf(about_text, sizeof(about_text),
+	      "GQ %s\n\n\n%s", VERSION, about_blurb);
      about_label = gtk_label_new(about_text);
      gtk_widget_show(about_label);
      gtk_box_pack_start(GTK_BOX(hbox), about_label, FALSE, FALSE, 4);
@@ -633,9 +648,14 @@ GtkWidget *create_about_window(void)
      gtk_container_border_width(GTK_CONTAINER(dialog_action_area2), 10);
      gtk_widget_show(dialog_action_area2);
 
-     about_ok_button = gtk_button_new_with_label("    OK    ");
+     hbox0 = gtk_hbutton_box_new();
+     gtk_widget_show(hbox0);
+     gtk_box_pack_start(GTK_BOX(dialog_action_area2), hbox0,
+			TRUE, FALSE, 0);
+
+     about_ok_button = gtk_button_new_with_label(_("OK"));
      gtk_widget_show(about_ok_button);
-     gtk_box_pack_start(GTK_BOX(dialog_action_area2), about_ok_button,
+     gtk_box_pack_start(GTK_BOX(hbox0), about_ok_button,
 			TRUE, FALSE, 0);
      GTK_WIDGET_SET_FLAGS(about_ok_button, GTK_CAN_DEFAULT);
      gtk_widget_grab_default(about_ok_button);
@@ -657,8 +677,8 @@ gboolean ctrl_b_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj)
 {
 
      if(event && event->type == GDK_KEY_PRESS &&
-	event->state & GDK_CONTROL_MASK &&
-	(event->keyval == GDK_b || event->keyval == GDK_c)) {
+	event->state & GDK_CONTROL_MASK && event->keyval == GDK_b) {
+	 /*	(event->keyval == GDK_b || event->keyval == GDK_c)) { */
 	  gtk_signal_emit_by_name(GTK_OBJECT(obj), "activate");
 	  gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
      }
