@@ -21,18 +21,30 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: mainwin.h,v 1.10 2002/06/13 10:03:33 stamfest Exp $ */
+
+#ifndef GQ_MAINWIN_H_INCLUDED
+#define GQ_MAINWIN_H_INCLUDED
+
 
 #include "config.h"
+#include <gtk/gtk.h>
 
 #define PACKAGEVERSION  PACKAGE " " VERSION
 #define about_blurb     "The gentleman's LDAP client" \
-                        "\n\n(C) 2002 Bert Vermeulen\nbert@biot.com" \
+                        "\n\n(C) 1998-2002 Bert Vermeulen <bert@biot.com>" \
+                        "\nand" \
+                        "\n(C) 2002 Peter Stamfest <peter@stamfest.at>" \
 			"\n\nhttp://biot.com/gq/"
 
 #define SEARCH_MODE       1
 #define BROWSE_MODE       2
 #define SCHEMA_MODE       3
 
+extern GList *tablist;
+extern GtkWidget *mainwin;
+extern GtkWidget *statusbar;
+
 void fill_serverlist_combo(GtkWidget *combo);
 void cleanup(void);
 void gq_exit(GtkWidget *widget, gpointer *data);
@@ -48,3 +60,13 @@ void create_license_window(void);
 void close_license_window(GtkWidget *widget);
 GtkWidget *create_about_window(void);
 gboolean ctrl_b_hack(GtkWidget *widget, GdkEventKey *event, gpointer obj);
+
+
+#endif
+
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
