@@ -21,6 +21,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: prefs.c,v 1.23 2002/07/03 20:53:16 stamfest Exp $ */
 
 #include <config.h>
 
@@ -41,6 +42,7 @@
 #include "template.h"
 #include "errorchain.h"
 #include "debug.h"
+#include "i18n.h"
 
 
 extern struct gq_config config;
@@ -57,11 +59,51 @@ void server_edit_callback(GtkWidget *this, gpointer data)
      GtkWidget *window, *field;
      struct ldapserver *server, *servers;
      int server_name_changed;
-     char *text;
+     char *text, *passwdtext, *passwdtext2;
+     gboolean passwdmatch;
+#if HAVE_LDAP_CLIENT_CACHE
+     char *ep;
+     int tmp;
+#endif
 
      server = data;
      window = current_edit_server_window;
 
+     /* Bind Password - retrieve asap, as we might pop up a warning
+        regarding cleartext passwords in .gq */
+     field = get_widget(window, "bindpw");
+     passwdtext = gtk_entry_get_text(GTK_ENTRY(field));
+
+     field = get_widget(window, "bindpw2");
+     passwdtext2 = gtk_entry_get_text(GTK_ENTRY(field));
+
+     if (passwdtext == NULL && passwdtext2 == NULL) {
+	  passwdmatch = TRUE;
+     } else if (passwdtext && passwdtext2) {
+	  passwdmatch = strcmp(passwdtext, passwdtext2) == 0;
+     } else { /* one is missing */
+	  passwdmatch = FALSE;
+     }
+
+     if (!passwdmatch) {
+	  single_warning_popup(_("Passwords do not match"));
+	  return;
+     }
+
+     if (passwdtext && strlen(passwdtext) > 0) {
+	  int rc = 
+	       question_popup(_("Warning: Cleartext password"),
+			      _("The password you have entered will be stored as clear text in your\n"
+				"HOME directory in the file .gq. This file will not be readable\n"
+				"by anybody else (except the system administrator), but you may\n"
+				"nevertheless prefer to not store your bind password as clear text.\n\n"
+				"Do you want to store your bind password as clear text?"));
+	  if (!rc) {
+	       return;
+	  }
+     }
+  
+
      /* Name */
      field = get_widget(window, "servername");
      text = gtk_entry_get_text(GTK_ENTRY(field));
@@ -77,9 +119,9 @@ void server_edit_callback(GtkWidget *this, gpointer data)
 	  if(server != servers) {
 	       if(!strcmp(servers->name, server->name)) {
 		    /* already exists */
-		    error_popup("Error adding new server",
-				"A server by that name already exists\n\n"
-				"Please choose another name");
+		    error_popup(_("Error adding new server"),
+				_("A server by that name already exists\n\n"
+				  "Please choose another name"));
 		    return;
 	       }
 	  }
@@ -107,9 +149,7 @@ void server_edit_callback(GtkWidget *this, gpointer data)
      strncpy(server->binddn, text, MAX_DN_LEN);
 
      /* Bind Password */
-     field = get_widget(window, "bindpw");
-     text = gtk_entry_get_text(GTK_ENTRY(field));
-     strncpy(server->bindpw, text, MAX_BINDPW_LEN);
+     strncpy(server->bindpw, passwdtext, MAX_BINDPW_LEN);
 
      /* Search attribute */
      field = get_widget(window, "searchattr");
@@ -121,6 +161,10 @@ void server_edit_callback(GtkWidget *this, gpointer data)
      text = gtk_entry_get_text(GTK_ENTRY(field));
      server->maxentries = atoi(text);
 
+     /* Ask password */
+     field = get_widget(window, "ask_pw");
+     server->ask_pw = GTK_TOGGLE_BUTTON(field)->active ? 1 : 0;
+
      /* Cache connection */
      field = get_widget(window, "cacheconn");
      server->cacheconn = GTK_TOGGLE_BUTTON(field)->active ? 1 : 0;
@@ -129,6 +173,14 @@ void server_edit_callback(GtkWidget *this, gpointer data)
      field = get_widget(window, "enabletls");
      server->enabletls = GTK_TOGGLE_BUTTON(field)->active ? 1 : 0;
 
+#if HAVE_LDAP_CLIENT_CACHE
+     /* Local Cache Timeout */
+     field = get_widget(window, "localcachetimeout");
+     text = gtk_entry_get_text(GTK_ENTRY(field));
+     tmp = (int) strtol(text, &ep, 0);
+     if (!ep || !*ep) server->local_cache_timeout = tmp;
+#endif
+
      /* Bind type */
      text = (char *) get_widget(window, "bindtype");
      server->bindtype = tokenize((struct tokenlist *)token_bindtype, text);
@@ -200,6 +252,15 @@ void bindtype_changed_callback(GtkEntry *entry, GtkWidget *window)
 
 }
 
+static void ask_pw_toggled(GtkToggleButton *button, GtkWidget *window)
+{
+     GtkWidget *field = get_widget(window, "bindpw");
+     gtk_widget_set_sensitive(field, !gtk_toggle_button_get_active(button));
+
+     field = get_widget(window, "bindpw2");
+     gtk_widget_set_sensitive(field, !gtk_toggle_button_get_active(button));
+}
+
 
 void create_edit_server_window(struct ldapserver *server)
 {
@@ -208,6 +269,7 @@ void create_edit_server_window(struct ldapserver *server)
      GtkWidget *table1, *table2;
      GtkWidget *label, *entry, *button;
      GtkWidget *okbutton, *cancelbutton;
+     GtkWidget *bindpw, *bindpw2;
      int y, editing_new_server;
      char title[MAX_SERVERNAME_LEN + 8];
      char tmp[16];
@@ -231,11 +293,11 @@ void create_edit_server_window(struct ldapserver *server)
      current_edit_server_window = editwindow;
 
      if(editing_new_server)
-	  sprintf(title, "New server");
+	  snprintf(title, sizeof(title), _("New server"));
      else
-	  sprintf(title, "Server %s", server->name);
+	  snprintf(title, sizeof(title), _("Server %s"), server->name);
      gtk_window_set_title(GTK_WINDOW(editwindow), title);
-     gtk_window_set_policy(GTK_WINDOW(editwindow), FALSE, FALSE, FALSE);
+     gtk_window_set_policy(GTK_WINDOW(editwindow), TRUE, TRUE, FALSE);
 
      main_vbox = gtk_vbox_new(FALSE, 0);
      gtk_container_border_width(GTK_CONTAINER(main_vbox), 10);
@@ -254,7 +316,7 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_object_set_data(GTK_OBJECT(editwindow), "vbox1", vbox1);
      gtk_widget_show(vbox1);
      gtk_container_border_width(GTK_CONTAINER(vbox1), 10);
-     label = gtk_label_new("General");
+     label = gtk_label_new(_("General"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox1, label);
 
@@ -267,7 +329,7 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_table_set_col_spacings(GTK_TABLE(table1), 13);
 
      /* Name */
-     label = gtk_label_new("Name");
+     label = gtk_label_new(_("Name"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -285,7 +347,7 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_widget_grab_focus(entry);
 
      /* LDAP host */
-     label = gtk_label_new("LDAP host");
+     label = gtk_label_new(_("LDAP host"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 1, 2,
@@ -301,7 +363,7 @@ void create_edit_server_window(struct ldapserver *server)
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
      /* Port */
-     label = gtk_label_new("LDAP Port");
+     label = gtk_label_new(_("LDAP Port"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
@@ -309,7 +371,7 @@ void create_edit_server_window(struct ldapserver *server)
 
      entry = gtk_entry_new();
      gtk_object_set_data(GTK_OBJECT(editwindow), "ldapport", entry);
-     sprintf(tmp, "%d", server->ldapport);
+     snprintf(tmp, sizeof(tmp), "%d", server->ldapport);
      gtk_entry_set_text(GTK_ENTRY(entry), tmp);
      gtk_widget_show(entry);
      gtk_signal_connect(GTK_OBJECT(entry), "activate",
@@ -318,7 +380,7 @@ void create_edit_server_window(struct ldapserver *server)
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
      /* Base DN */
-     label = gtk_label_new("Base DN");
+     label = gtk_label_new(_("Base DN"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 3, 4,
@@ -340,7 +402,7 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_object_set_data(GTK_OBJECT(editwindow), "vbox2", vbox2);
      gtk_widget_show(vbox2);
      gtk_container_border_width(GTK_CONTAINER(vbox2), 10);
-     label = gtk_label_new("Details");
+     label = gtk_label_new(_("Details"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, label);
 
@@ -354,7 +416,7 @@ void create_edit_server_window(struct ldapserver *server)
      y = 0;
 
      /* Bind DN */
-     label = gtk_label_new("Bind DN");
+     label = gtk_label_new(_("Bind DN"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
@@ -371,13 +433,17 @@ void create_edit_server_window(struct ldapserver *server)
      y++;
 
      /* Bind Password */
-     label = gtk_label_new("Bind Password");
+     label = gtk_label_new(_("Bind Password"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
-     entry = gtk_entry_new();
+     bindpw = entry = gtk_entry_new();
+     /* Follow the suggestion of Jacek Konieczny <jajcus@bnet.pl>,
+	(even though Bert won't like it[?]) and do not show the password
+	in cleartext... */
+     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
      gtk_object_set_data(GTK_OBJECT(editwindow), "bindpw", entry);
      gtk_entry_set_text(GTK_ENTRY(entry), server->bindpw);
      gtk_widget_show(entry);
@@ -387,8 +453,29 @@ void create_edit_server_window(struct ldapserver *server)
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
      y++;
 
+     /* Bind Password (again)*/
+     label = gtk_label_new(_("Bind Password (again)"));
+     gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
+     gtk_widget_show(label);
+     gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
+		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+
+     bindpw2 = entry = gtk_entry_new();
+     /* Follow the suggestion of Jacek Konieczny <jajcus@bnet.pl>,
+	(even though Bert won't like it[?]) and do not show the password
+	in cleartext... */
+     gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+     gtk_object_set_data(GTK_OBJECT(editwindow), "bindpw2", entry);
+     gtk_entry_set_text(GTK_ENTRY(entry), server->bindpw);
+     gtk_widget_show(entry);
+     gtk_signal_connect(GTK_OBJECT(entry), "activate",
+			GTK_SIGNAL_FUNC(server_edit_callback), server);
+     gtk_table_attach(GTK_TABLE(table2), entry, 1, 2, y, y + 1,
+		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+     y++;
+
      /* Bind type */
-     label = gtk_label_new("Bind type");
+     label = gtk_label_new(_("Bind type"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
@@ -399,9 +486,13 @@ void create_edit_server_window(struct ldapserver *server)
 						BINDTYPE_SIMPLE));
      bindtypes = g_list_append(bindtypes, detokenize((struct tokenlist *) token_bindtype,
 						     BINDTYPE_KERBEROS));
+     bindtypes = g_list_append(bindtypes, detokenize((struct tokenlist *) token_bindtype,
+						     BINDTYPE_SASL));
      gtk_object_set_data(GTK_OBJECT(editwindow), "bindtype",
 			 detokenize((struct tokenlist *) token_bindtype, BINDTYPE_SIMPLE));
      gtk_combo_set_popdown_strings(GTK_COMBO(bindtype), bindtypes);
+     g_list_free(bindtypes);
+
      gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(bindtype)->entry), FALSE);
      gtk_signal_connect(GTK_OBJECT(GTK_COMBO(bindtype)->entry), "changed",
 			GTK_SIGNAL_FUNC(bindtype_changed_callback),
@@ -413,7 +504,7 @@ void create_edit_server_window(struct ldapserver *server)
      y++;
 
      /* Search attribute */
-     label = gtk_label_new("Search Attribute");
+     label = gtk_label_new(_("Search Attribute"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
@@ -430,7 +521,7 @@ void create_edit_server_window(struct ldapserver *server)
      y++;
 
      /* Maximum entries */
-     label = gtk_label_new("Maximum entries");
+     label = gtk_label_new(_("Maximum entries"));
      gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
@@ -438,16 +529,52 @@ void create_edit_server_window(struct ldapserver *server)
 
      entry = gtk_entry_new();
      gtk_object_set_data(GTK_OBJECT(editwindow), "maxentries", entry);
-     sprintf(tmp, "%d", server->maxentries);
+     snprintf(tmp, sizeof(tmp), "%d", server->maxentries);
      gtk_entry_set_text(GTK_ENTRY(entry), tmp);
      gtk_widget_show(entry);
      gtk_signal_connect(GTK_OBJECT(entry), "activate",
 			GTK_SIGNAL_FUNC(server_edit_callback), server);
      gtk_table_attach(GTK_TABLE(table2), entry, 1, 2, y, y + 1,
 		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+     y++;
+
+#if HAVE_LDAP_CLIENT_CACHE
+     /* Use local cache */
+     label = gtk_label_new(_("LDAP cache timeout"));
+     gtk_misc_set_alignment(GTK_MISC(label), 0.0, .5);
+     gtk_widget_show(label);
+     gtk_table_attach(GTK_TABLE(table2), label, 0, 1, y, y + 1,
+		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+
+     entry = gtk_entry_new();
+     gtk_object_set_data(GTK_OBJECT(editwindow), "localcachetimeout", entry);
+     snprintf(tmp, sizeof(tmp), "%ld", server->local_cache_timeout);
+     gtk_entry_set_text(GTK_ENTRY(entry), tmp);
+     gtk_widget_show(entry);
+     gtk_signal_connect(GTK_OBJECT(entry), "activate",
+			GTK_SIGNAL_FUNC(server_edit_callback), server);
+     gtk_table_attach(GTK_TABLE(table2), entry, 1, 2, y, y + 1,
+		      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+#endif
+
+     /* Ask password on first connect */
+     button = gtk_check_button_new_with_label(_("Ask password on first connect"));
+     gtk_object_set_data(GTK_OBJECT(editwindow), "ask_pw", button);
+     if(server->ask_pw) {
+	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
+	  gtk_widget_set_sensitive(GTK_WIDGET(bindpw), FALSE);
+	  gtk_widget_set_sensitive(GTK_WIDGET(bindpw2), FALSE);
+     }
+
+     gtk_signal_connect(GTK_OBJECT(button), "toggled",
+			GTK_SIGNAL_FUNC(ask_pw_toggled), bindpw);
+
+     GTK_WIDGET_UNSET_FLAGS(GTK_CHECK_BUTTON(button), GTK_CAN_FOCUS);
+     gtk_widget_show(button);
+     gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, TRUE, 2);
 
      /* Cache connections */
-     button = gtk_check_button_new_with_label("Cache connection");
+     button = gtk_check_button_new_with_label(_("Cache connection"));
      gtk_object_set_data(GTK_OBJECT(editwindow), "cacheconn", button);
      if(server->cacheconn)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
@@ -456,7 +583,7 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, TRUE, 2);
 
      /* Enable TLS */
-     button = gtk_check_button_new_with_label("Enable TLS");
+     button = gtk_check_button_new_with_label(_("Enable TLS"));
      gtk_object_set_data(GTK_OBJECT(editwindow), "enabletls", button);
      if(server->enabletls)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button), TRUE);
@@ -464,14 +591,13 @@ void create_edit_server_window(struct ldapserver *server)
      gtk_widget_show(button);
      gtk_box_pack_start(GTK_BOX(vbox2), button, FALSE, TRUE, 0);
 
-
      /* OK and Cancel buttons */
-     hbox = gtk_hbox_new(FALSE, 13);
-     gtk_container_border_width(GTK_CONTAINER(hbox), 10);
+     hbox = gtk_hbutton_box_new(); /* FALSE, 13); */
+/*       gtk_container_border_width(GTK_CONTAINER(hbox), 10); */
      gtk_widget_show(hbox);
-     gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, TRUE, 0);
+     gtk_box_pack_start(GTK_BOX(main_vbox), hbox, FALSE, TRUE, 10);
 
-     okbutton = gtk_button_new_with_label("  OK  ");
+     okbutton = gtk_button_new_with_label(_("OK"));
      gtk_widget_show(okbutton);
      gtk_signal_connect(GTK_OBJECT(okbutton), "clicked",
 			GTK_SIGNAL_FUNC(server_edit_callback),
@@ -480,7 +606,7 @@ void create_edit_server_window(struct ldapserver *server)
      GTK_WIDGET_SET_FLAGS(okbutton, GTK_CAN_DEFAULT);
      gtk_widget_grab_default(okbutton);
 
-     cancelbutton = gtk_button_new_with_label(" Cancel ");
+     cancelbutton = gtk_button_new_with_label(_("Cancel"));
      gtk_widget_show(cancelbutton);
      if(editing_new_server) {
 	  gtk_signal_connect_object(GTK_OBJECT(cancelbutton), "clicked",
@@ -676,10 +802,10 @@ void create_prefs_window(void)
      GtkWidget *hbox_buttons, *okbutton, *cancelbutton;
 
      prefswindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-     gtk_widget_set_usize(prefswindow, 420, 370);
+     gtk_widget_set_usize(prefswindow, 520, 470);
      gtk_container_border_width(GTK_CONTAINER(prefswindow), 5);
-     gtk_window_set_title(GTK_WINDOW(prefswindow), "Preferences");
-     gtk_window_set_policy(GTK_WINDOW(prefswindow), FALSE, FALSE, FALSE);
+     gtk_window_set_title(GTK_WINDOW(prefswindow), _("Preferences"));
+     gtk_window_set_policy(GTK_WINDOW(prefswindow), TRUE, TRUE, FALSE);
      gtk_signal_connect_object(GTK_OBJECT(prefswindow), "key_press_event",
 			       GTK_SIGNAL_FUNC(close_on_esc),
 			       (gpointer) prefswindow);
@@ -697,7 +823,7 @@ void create_prefs_window(void)
      gtk_container_border_width(GTK_CONTAINER(vbox_search_options), 10);
      create_search_optionstab(vbox_search_options, prefswindow);
      gtk_widget_show(vbox_search_options);
-     label = gtk_label_new("Search Options");
+     label = gtk_label_new(_("Search Options"));
      gtk_widget_show(label);
      GTK_WIDGET_UNSET_FLAGS(GTK_NOTEBOOK(notebook), GTK_CAN_FOCUS);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_search_options, label);
@@ -707,7 +833,7 @@ void create_prefs_window(void)
      gtk_container_border_width(GTK_CONTAINER(vbox_browse_options), 10);
      create_browse_optionstab(vbox_browse_options, prefswindow);
      gtk_widget_show(vbox_browse_options);
-     label = gtk_label_new("Browse Options");
+     label = gtk_label_new(_("Browse Options"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_browse_options, label);
 
@@ -715,7 +841,7 @@ void create_prefs_window(void)
      vbox_servers = gtk_vbox_new(FALSE, 0);
      create_serverstab(vbox_servers);
      gtk_widget_show(vbox_servers);
-     label = gtk_label_new("Servers");
+     label = gtk_label_new(_("Servers"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_servers, label);
 
@@ -724,7 +850,7 @@ void create_prefs_window(void)
      vbox_templates = gtk_vbox_new(FALSE, 0);
      create_templatestab(vbox_templates, prefswindow);
      gtk_widget_show(vbox_templates);
-     label = gtk_label_new("Templates");
+     label = gtk_label_new(_("Templates"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_templates, label);
 #else
@@ -736,17 +862,17 @@ void create_prefs_window(void)
      gtk_container_border_width(GTK_CONTAINER(vbox_ldif), 10);
      create_ldiftab(vbox_ldif, prefswindow);
      gtk_widget_show(vbox_ldif);
-     label = gtk_label_new("LDIF");
+     label = gtk_label_new(_("LDIF"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox_ldif, label);
 
      /* OK and Cancel buttons outside notebook */
-     hbox_buttons = gtk_hbox_new(FALSE, 0);
+     hbox_buttons = gtk_hbutton_box_new(); /* FALSE, 0); */
      gtk_widget_show(hbox_buttons);
-     gtk_box_pack_start(GTK_BOX(vbox2), hbox_buttons, FALSE, TRUE, 0);
-     gtk_container_border_width(GTK_CONTAINER(hbox_buttons), 10);
+     gtk_box_pack_start(GTK_BOX(vbox2), hbox_buttons, FALSE, TRUE, 10);
+/*       gtk_container_border_width(GTK_CONTAINER(hbox_buttons), 10); */
 
-     okbutton = gtk_button_new_with_label("  OK  ");
+     okbutton = gtk_button_new_with_label(_("OK"));
      gtk_widget_show(okbutton);
      gtk_signal_connect_object(GTK_OBJECT(okbutton), "pressed",
 			       GTK_SIGNAL_FUNC(prefs_okbutton_callback),
@@ -756,7 +882,7 @@ void create_prefs_window(void)
      gtk_widget_grab_focus(okbutton);
      gtk_widget_grab_default(okbutton);
 
-     cancelbutton = gtk_button_new_with_label("  Cancel  ");
+     cancelbutton = gtk_button_new_with_label(_("Cancel"));
      gtk_widget_show(cancelbutton);
      gtk_box_pack_end(GTK_BOX(hbox_buttons), cancelbutton, FALSE, TRUE, 0);
      gtk_signal_connect_object(GTK_OBJECT(cancelbutton), "pressed",
@@ -770,7 +896,7 @@ void create_prefs_window(void)
 
 void create_serverstab(GtkWidget *target)
 {
-     GtkWidget *vbox1, *vbox2, *hbox1, *hbox2, *scrwin;
+     GtkWidget *vbox0, *vbox1, *vbox2, *hbox1, *hbox2, *scrwin;
      GtkWidget *button_new, *button_edit, *button_delete;
      GtkWidget *server_clist;
 
@@ -805,12 +931,16 @@ void create_serverstab(GtkWidget *target)
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrwin), server_clist);
      fill_serverlist_serverstab();
 
-     vbox2 = gtk_vbox_new(FALSE, 10);
+     vbox0 = gtk_vbox_new(FALSE, 10);
+     gtk_widget_show(vbox0);
+     gtk_box_pack_start(GTK_BOX(hbox2), vbox0, FALSE, FALSE, 0);
+
+     vbox2 = gtk_vbutton_box_new(); /*FALSE, 10); */
      gtk_widget_show(vbox2);
-     gtk_box_pack_start(GTK_BOX(hbox2), vbox2, TRUE, TRUE, 0);
+     gtk_box_pack_start(GTK_BOX(vbox0), vbox2, FALSE, FALSE, 0);
 
      /* New button */
-     button_new = gtk_button_new_with_label("New");
+     button_new = gtk_button_new_with_label(_("New"));
      GTK_WIDGET_UNSET_FLAGS(button_new, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_new", button_new);
      gtk_widget_show(button_new);
@@ -820,7 +950,7 @@ void create_serverstab(GtkWidget *target)
      gtk_box_pack_start(GTK_BOX(vbox2), button_new, FALSE, TRUE, 0);
 
      /* Edit button */
-     button_edit = gtk_button_new_with_label("Edit");
+     button_edit = gtk_button_new_with_label(_("Edit"));
      GTK_WIDGET_UNSET_FLAGS(button_edit, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_edit", button_edit);
      gtk_widget_show(button_edit);
@@ -830,7 +960,7 @@ void create_serverstab(GtkWidget *target)
      gtk_box_pack_start(GTK_BOX(vbox2), button_edit, FALSE, TRUE, 0);
 
      /* Delete button */
-     button_delete = gtk_button_new_with_label("Delete");
+     button_delete = gtk_button_new_with_label(_("Delete"));
      GTK_WIDGET_UNSET_FLAGS(button_delete, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_delete", button_delete);
      gtk_widget_show(button_delete);
@@ -850,7 +980,7 @@ void create_serverstab(GtkWidget *target)
 #ifdef HAVE_LDAP_STR2OBJECTCLASS
 void create_templatestab(GtkWidget *target, GtkWidget *targetwindow)
 {
-     GtkWidget *vbox1, *vbox2, *hbox1, *hbox2, *scrwin;
+     GtkWidget *vbox0, *vbox1, *vbox2, *hbox1, *hbox2, *scrwin;
      GtkWidget *button_new, *button_edit, *button_delete;
      GtkWidget *template_clist, *label, *combo;
 
@@ -865,7 +995,7 @@ void create_templatestab(GtkWidget *target, GtkWidget *targetwindow)
 
 
      /* Schema server */
-     label = gtk_label_new("Last resort schema server");
+     label = gtk_label_new(_("Last resort schema server"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 0);
 
@@ -904,12 +1034,16 @@ void create_templatestab(GtkWidget *target, GtkWidget *targetwindow)
 
      fill_clist_templates(template_clist);
 
-     vbox2 = gtk_vbox_new(FALSE, 10);
+     vbox0 = gtk_vbox_new(FALSE, 10);
+     gtk_widget_show(vbox0);
+     gtk_box_pack_start(GTK_BOX(hbox2), vbox0, FALSE, FALSE, 0);
+
+     vbox2 = gtk_vbutton_box_new(); /*FALSE, 10); */
      gtk_widget_show(vbox2);
-     gtk_box_pack_start(GTK_BOX(hbox2), vbox2, TRUE, TRUE, 0);
+     gtk_box_pack_start(GTK_BOX(vbox0), vbox2, FALSE, FALSE, 0);
 
      /* New button */
-     button_new = gtk_button_new_with_label("New");
+     button_new = gtk_button_new_with_label(_("New"));
      GTK_WIDGET_UNSET_FLAGS(button_new, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_new", button_new);
      gtk_widget_show(button_new);
@@ -919,7 +1053,7 @@ void create_templatestab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox2), button_new, FALSE, TRUE, 0);
 
      /* Edit button */
-     button_edit = gtk_button_new_with_label("Edit");
+     button_edit = gtk_button_new_with_label(_("Edit"));
      GTK_WIDGET_UNSET_FLAGS(button_edit, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_edit", button_edit);
      gtk_widget_show(button_edit);
@@ -929,7 +1063,7 @@ void create_templatestab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox2), button_edit, FALSE, TRUE, 0);
 
      /* Delete button */
-     button_delete = gtk_button_new_with_label("Delete");
+     button_delete = gtk_button_new_with_label(_("Delete"));
      GTK_WIDGET_UNSET_FLAGS(button_delete, GTK_CAN_FOCUS);
      gtk_object_set_data(GTK_OBJECT(target), "button_delete", button_delete);
      gtk_widget_show(button_delete);
@@ -954,7 +1088,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(target), hbox_options, FALSE, TRUE, 5);
 
      /* Search type frame in Options tab */
-     stframe = gtk_frame_new(" Search type ");
+     stframe = gtk_frame_new(_(" Search type "));
      gtk_widget_show(stframe);
      gtk_box_pack_start(GTK_BOX(hbox_options), stframe, FALSE, TRUE, 0);
      vbox_st = gtk_vbox_new(TRUE, 0);
@@ -962,7 +1096,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_widget_show(vbox_st);
      gtk_container_add(GTK_CONTAINER(stframe), vbox_st);
 
-     stradiobutton = gtk_radio_button_new_with_label(NULL, "Begins with");
+     stradiobutton = gtk_radio_button_new_with_label(NULL, _("Begins with"));
      if(config.search_argument == SEARCHARG_BEGINS_WITH)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stradiobutton), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(stradiobutton), GTK_CAN_FOCUS);
@@ -971,7 +1105,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_widget_show(stradiobutton);
 
      stgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(stradiobutton));
-     stradiobutton = gtk_radio_button_new_with_label(stgroup, "Ends with");
+     stradiobutton = gtk_radio_button_new_with_label(stgroup, _("Ends with"));
      if(config.search_argument == SEARCHARG_ENDS_WITH)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stradiobutton), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(stradiobutton), GTK_CAN_FOCUS);
@@ -980,7 +1114,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_widget_show(stradiobutton);
 
      stgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(stradiobutton));
-     stradiobutton = gtk_radio_button_new_with_label(stgroup, "Contains");
+     stradiobutton = gtk_radio_button_new_with_label(stgroup, _("Contains"));
      if(config.search_argument == SEARCHARG_CONTAINS)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stradiobutton), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(stradiobutton), GTK_CAN_FOCUS);
@@ -989,7 +1123,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_widget_show(stradiobutton);
 
      stgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(stradiobutton));
-     stradiobutton = gtk_radio_button_new_with_label(stgroup, "Equals");
+     stradiobutton = gtk_radio_button_new_with_label(stgroup, _("Equals"));
      if(config.search_argument == SEARCHARG_EQUALS)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(stradiobutton), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(stradiobutton), GTK_CAN_FOCUS);
@@ -999,7 +1133,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
 
 #if 0
      /* Sorting frame in Options tab */
-     sortframe = gtk_frame_new(" Sorting ");
+     sortframe = gtk_frame_new(_(" Sorting "));
      gtk_widget_show(sortframe);
      gtk_box_pack_start(GTK_BOX(hbox_options), sortframe, FALSE, TRUE, 0);
      vbox_sort = gtk_vbox_new(FALSE, 0);
@@ -1008,7 +1142,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_container_add(GTK_CONTAINER(sortframe), vbox_sort);
 
      /* Sort in search mode button */
-     sort_search_button = gtk_check_button_new_with_label("Search mode");
+     sort_search_button = gtk_check_button_new_with_label(_("Search mode"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "sort_search", sort_search_button);
      if(config.sort_search)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sort_search_button), TRUE);
@@ -1017,7 +1151,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox_sort), sort_search_button, FALSE, FALSE, 5);
 #endif
      /* View frame in Options tab */
-     viewframe = gtk_frame_new(" View ");
+     viewframe = gtk_frame_new(_(" View "));
      gtk_widget_show(viewframe);
      gtk_box_pack_start(GTK_BOX(target), viewframe, TRUE, TRUE, 5);
      vbox_view = gtk_vbox_new(TRUE, 0);
@@ -1026,7 +1160,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_container_add(GTK_CONTAINER(viewframe), vbox_view);
 
      /* Show Distinguished Name checkbox */
-     dnbutton = gtk_check_button_new_with_label("Show Distinguished Name");
+     dnbutton = gtk_check_button_new_with_label(_("Show Distinguished Name"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "showdn", dnbutton);
      if(config.showdn)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(dnbutton), TRUE);
@@ -1035,7 +1169,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox_view), dnbutton, FALSE, TRUE, 5);
 
      /* Show Objectclass checkbox */
-     ocbutton = gtk_check_button_new_with_label("Show Objectclass");
+     ocbutton = gtk_check_button_new_with_label(_("Show Objectclass"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "showoc", ocbutton);
      if(config.showoc)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(ocbutton), TRUE);
@@ -1044,7 +1178,7 @@ void create_search_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox_view), ocbutton, FALSE, TRUE, 5);
 
      /* Sort in search mode button */
-     sort_search_button = gtk_check_button_new_with_label("Sort results");
+     sort_search_button = gtk_check_button_new_with_label(_("Sort results"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "sort_search", sort_search_button);
      if(config.sort_search)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sort_search_button), TRUE);
@@ -1060,7 +1194,7 @@ void create_browse_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      GtkWidget *show_rdn_only_button, *sort_browse_button;
 
   /* View frame in Options tab */
-     viewframe = gtk_frame_new(" View ");
+     viewframe = gtk_frame_new(_(" View "));
      gtk_widget_show(viewframe);
      gtk_box_pack_start(GTK_BOX(target), viewframe, FALSE, TRUE, 5);
      vbox_view = gtk_vbox_new(TRUE, 0);
@@ -1069,7 +1203,7 @@ void create_browse_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_container_add(GTK_CONTAINER(viewframe), vbox_view);
 
   /* show rdn only button */
-     show_rdn_only_button = gtk_check_button_new_with_label("Show Relative Distinguished Name only");
+     show_rdn_only_button = gtk_check_button_new_with_label(_("Show Relative Distinguished Name only"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "show_rdn_only", show_rdn_only_button);
      if(config.show_rdn_only)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(show_rdn_only_button), TRUE);
@@ -1078,7 +1212,7 @@ void create_browse_optionstab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_box_pack_start(GTK_BOX(vbox_view), show_rdn_only_button, FALSE, FALSE, 5);
 
   /* Sort in browse mode button */
-     sort_browse_button = gtk_check_button_new_with_label("Sort results");
+     sort_browse_button = gtk_check_button_new_with_label(_("Sort results"));
      gtk_object_set_data(GTK_OBJECT(targetwindow), "sort_browse", sort_browse_button);
      if(config.sort_browse)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(sort_browse_button), TRUE);
@@ -1098,7 +1232,7 @@ void create_ldiftab(GtkWidget *target, GtkWidget *targetwindow)
      GtkWidget *vbox1;
 
      /* Format frame */
-     formatframe = gtk_frame_new(" Format ");
+     formatframe = gtk_frame_new(_(" Format "));
      gtk_widget_show(formatframe);
      gtk_box_pack_start(GTK_BOX(target), formatframe, FALSE, TRUE, 5);
 
@@ -1107,7 +1241,7 @@ void create_ldiftab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_container_add(GTK_CONTAINER(formatframe), vbox1);
      gtk_widget_show(vbox1);
 
-     formatradio = gtk_radio_button_new_with_label(NULL, "UMich/OpenLDAP style (no comments/version)");
+     formatradio = gtk_radio_button_new_with_label(NULL, _("UMich/OpenLDAP style (no comments/version)"));
      if(config.ldifformat == LDIF_UMICH)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(formatradio), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(formatradio), GTK_CAN_FOCUS);
@@ -1116,7 +1250,7 @@ void create_ldiftab(GtkWidget *target, GtkWidget *targetwindow)
      gtk_widget_show(formatradio);
 
      formatgroup = gtk_radio_button_group(GTK_RADIO_BUTTON(formatradio));
-     formatradio = gtk_radio_button_new_with_label(formatgroup, "LDIF Version 1 (RFC2849)");
+     formatradio = gtk_radio_button_new_with_label(formatgroup, _("LDIF Version 1 (RFC2849)"));
      if(config.ldifformat == LDIF_V1)
 	  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(formatradio), TRUE);
      GTK_WIDGET_UNSET_FLAGS(GTK_RADIO_BUTTON(formatradio), GTK_CAN_FOCUS);
@@ -1223,3 +1357,9 @@ GtkWidget* get_widget(GtkWidget *widget, gchar *widget_name)
      return found_widget;
 }
 
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
