@@ -37,6 +37,7 @@
 #include "configfile.h"
 #include "util.h"
 #include "schema.h"
+#include "i18n.h"
 #include "debug.h"
 
 extern struct gq_config config;
@@ -84,7 +85,7 @@ struct server_schema *get_schema(struct ldapserver *server)
 
 struct server_schema *get_server_schema(struct ldapserver *server)
 {
-     BerElement *ptr;
+     BerElement *berptr;
      LDAP *ld;
      LDAPMessage *res, *e; 
      LDAPObjectClass *oc;
@@ -114,67 +115,80 @@ struct server_schema *get_server_schema(struct ldapserver *server)
      ss = NULL;
      subschema[0] = 0;
      server->flags |= SERVER_HAS_NO_SCHEMA;
-     statusbar_msg("subschemaSubentry search on root DSE");
+     statusbar_msg(_("subschemaSubentry search on root DSE"));
      msg = ldap_search_s(ld, "", LDAP_SCOPE_BASE, "(objectclass=*)",
 			 subschemasubentry, 0, &res);
 
      if(msg != LDAP_SUCCESS) {
+	  if (msg == LDAP_SERVER_DOWN) {
+	       server->server_down++;
+	  }
 	  statusbar_msg(ldap_err2string(msg));
 	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
      if(res == NULL) {
-	  statusbar_msg("no schema information found");
+	  statusbar_msg(_("no schema information found"));
 	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
      if( (e = ldap_first_entry(ld, res)) ) {
-	  if( (attr = ldap_first_attribute(ld, res, &ptr)) ) {
+	  if( (attr = ldap_first_attribute(ld, res, &berptr)) ) {
 	       if( (vals = ldap_get_values(ld, res, attr)) ) {
-		    if(strlen(vals[0]) < 59) {
-			 strcpy(subschema, vals[0]);
-			 ldap_value_free(vals);
+		    if(strlen(vals[0]) < sizeof(subschema) - 1) {
+			 strcpy(subschema, vals[0]); /* Flawfinder: ignore */
 		    }
+		    ldap_value_free(vals);
 	       }
+	       ldap_memfree(attr);
 	  }
+#ifndef HAVE_OPENLDAP_12
+	  if(berptr)
+	       ber_free(berptr, 0);
+#endif
      }
      ldap_msgfree(res);
 
      if(subschema[0] == 0) {
-	  statusbar_msg("no schema information found");
+	  statusbar_msg(_("no schema information found"));
 	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
-     sprintf(message, "Schema search on %s", subschema);
+     snprintf(message, sizeof(message), _("Schema search on %s"), subschema);
      statusbar_msg(message);
      msg = ldap_search_s(ld, subschema, LDAP_SCOPE_BASE,
 			 "(objectclass=*)", schema_attrs, 0, &res);
 
      if(msg != LDAP_SUCCESS) {
+	  if (msg == LDAP_SERVER_DOWN) {
+	       server->server_down++;
+	  }
 	  statusbar_msg(ldap_err2string(msg));
 	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
      if(res == NULL) {
-	  statusbar_msg("no schema information found");
+	  statusbar_msg(_("no schema information found"));
 	  close_connection(server, FALSE);
 	  return(NULL);
      }
 
      ss = MALLOC(sizeof(struct server_schema), "struct server_schema");
 
-     if(ss == NULL)
+     if(ss == NULL) {
+	  close_connection(server, FALSE);
 	  return(NULL);
+     }
 
      ss->oc = ss->at = ss->mr = ss->s = NULL;
 
      for(e = ldap_first_entry(ld, res); e; e = ldap_next_entry(ld, e)) {
-	  for(attr = ldap_first_attribute(ld, res, &ptr); attr;
-	      attr = ldap_next_attribute(ld, res, ptr)) {
+	  for(attr = ldap_first_attribute(ld, res, &berptr); attr;
+	      attr = ldap_next_attribute(ld, res, berptr)) {
 
 	       vals = ldap_get_values(ld, res, attr);
 	       if(vals) {
@@ -206,8 +220,12 @@ struct server_schema *get_server_schema(struct ldapserver *server)
 		    }
 		    ldap_value_free(vals);
 	       }
-
+	       ldap_memfree(attr);
 	  }
+#ifndef HAVE_OPENLDAP12
+	  if(berptr)
+	       ber_free(berptr, 0);
+#endif
      }
      ldap_msgfree(res);
 
@@ -245,9 +263,9 @@ struct server_schema *get_server_schema(struct ldapserver *server)
 int sort_oc(LDAPObjectClass *oc1, LDAPObjectClass *oc2)
 {
 
-     if(oc1->oc_names[0] && oc2->oc_names[0])
+     if(oc1->oc_names && oc2->oc_names)
 	  return(strcasecmp(oc1->oc_names[0], oc2->oc_names[0]));
-     else
+     else if (oc1->oc_oid && oc2->oc_oid)
 	  return(strcasecmp(oc1->oc_oid, oc2->oc_oid));
 
      return(0);
@@ -327,4 +345,43 @@ LDAPObjectClass *find_oc_by_oc_name(struct server_schema *ss, char *ocname)
 }
 
 
+GList *attrlist_by_oclist(struct ldapserver *server, GList *oclist)
+{
+     GList *attrlist;
+     LDAPObjectClass *oc;
+     struct server_schema *ss;
+     int i;
+     char *objectclass;
+
+     ss = server->ss;
+     if(!ss || !ss->oc || !ss->at)
+	  return(NULL);
+
+     attrlist = NULL;
+     while(oclist) {
+	  objectclass = oclist->data;
+	  oc = find_oc_by_oc_name(ss, objectclass);
+	  if(oc) {
+	       if(oc->oc_at_oids_must)
+		    for(i = 0; oc->oc_at_oids_must[i]; i++)
+			 attrlist = g_list_append(attrlist, oc->oc_at_oids_must[i]);
+	       if(oc->oc_at_oids_may)
+		    for(i = 0; oc->oc_at_oids_may[i]; i++)
+			 attrlist = g_list_append(attrlist, oc->oc_at_oids_may[i]);
+
+	  }
+
+	  oclist = oclist->next;
+     }
+
+     return(attrlist);
+}
+
+
 #endif  /* HAVE_LDAP_STR2OBJECTCLASS */
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
