@@ -41,6 +41,7 @@
 #include "util.h"
 #include "schemabrowse.h"
 #include "schema.h"
+#include "i18n.h"
 #include "debug.h"
 
 
@@ -129,7 +130,8 @@ void add_schema_servers(GHashTable *hash, GtkWidget *treeroot)
 	  server_cnt++;
      }
 
-     make_message(message, server_cnt, "server", "servers", "found");
+     make_message(message, sizeof(message), 
+		  server_cnt, _("server"), _("servers"), _("found"));
      statusbar_msg(message);
 
 }
@@ -145,6 +147,7 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
      GtkWidget *new_item, *server_subtree, *new_subtree;
      struct ldapserver *server;
      struct server_schema *ss;
+     int i;
 
      gtk_signal_disconnect_by_func(GTK_OBJECT(item),
 				   GTK_SIGNAL_FUNC(attach_server_schema), hash);
@@ -178,15 +181,10 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
 	       while(tmp) {
 		    oc = (LDAPObjectClass *) tmp->data;
 		    if(oc->oc_names && oc->oc_names[0])
-			 new_item = gtk_tree_item_new_with_label(oc->oc_names[0]);
+			 for(i = 0; oc->oc_names[i]; i++)
+			      attach_schema_item(new_subtree, "oc", oc, oc->oc_names[i]);
 		    else
-			 new_item = gtk_tree_item_new_with_label(oc->oc_oid);
-		    GTK_WIDGET_UNSET_FLAGS(new_item, GTK_CAN_FOCUS);
-		    gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
-				       GTK_SIGNAL_FUNC(schema_button_tree), NULL);
-		    gtk_object_set_data(GTK_OBJECT(new_item), "oc", oc);
-		    gtk_widget_show(new_item);
-		    gtk_tree_append(GTK_TREE(new_subtree), new_item);
+			 attach_schema_item(new_subtree, "oc", oc, oc->oc_oid);
 		    tmp = tmp->next;
 	       }
 	  }
@@ -211,15 +209,10 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
 	       while(tmp) {
 		    at = (LDAPAttributeType *) tmp->data;
 		    if(at->at_names && at->at_names[0])
-			 new_item = gtk_tree_item_new_with_label(at->at_names[0]);
+			 for(i = 0; at->at_names[i]; i++)
+			      attach_schema_item(new_subtree, "at", at, at->at_names[i]);
 		    else
-			 new_item = gtk_tree_item_new_with_label(at->at_oid);
-		    GTK_WIDGET_UNSET_FLAGS(new_item, GTK_CAN_FOCUS);
-		    gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
-				       GTK_SIGNAL_FUNC(schema_button_tree), NULL);
-		    gtk_object_set_data(GTK_OBJECT(new_item), "at", at);
-		    gtk_widget_show(new_item);
-		    gtk_tree_append(GTK_TREE(new_subtree), new_item);
+			 attach_schema_item(new_subtree, "at", at, at->at_oid);
 		    tmp = tmp->next;
 	       }
 	  }
@@ -244,18 +237,12 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
 	       while(tmp) {
 		    mr = (LDAPMatchingRule *) tmp->data;
 		    if(mr->mr_names && mr->mr_names[0])
-			 new_item = gtk_tree_item_new_with_label(mr->mr_names[0]);
+			 for(i = 0; mr->mr_names[i]; i++)
+			      attach_schema_item(new_subtree, "mr", mr, mr->mr_names[i]);
 		    else
-			 new_item = gtk_tree_item_new_with_label(mr->mr_oid);
-		    GTK_WIDGET_UNSET_FLAGS(new_item, GTK_CAN_FOCUS);
-		    gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
-				       GTK_SIGNAL_FUNC(schema_button_tree), NULL);
-		    gtk_object_set_data(GTK_OBJECT(new_item), "mr", mr);
-		    gtk_widget_show(new_item);
-		    gtk_tree_append(GTK_TREE(new_subtree), new_item);
+			 attach_schema_item(new_subtree, "mr", mr, mr->mr_oid);
 		    tmp = tmp->next;
 	       }
-
 	  }
 
 	  if(ss->s) {
@@ -277,13 +264,7 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
 	       tmp = ss->s;
 	       while(tmp) {
 		    s = (LDAPSyntax *) tmp->data;
-		    new_item = gtk_tree_item_new_with_label(s->syn_oid);
-		    GTK_WIDGET_UNSET_FLAGS(new_item, GTK_CAN_FOCUS);
-		    gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
-				       GTK_SIGNAL_FUNC(schema_button_tree), NULL);
-		    gtk_widget_show(new_item);
-		    gtk_object_set_data(GTK_OBJECT(new_item), "s", s);
-		    gtk_tree_append(GTK_TREE(new_subtree), new_item);
+		    attach_schema_item(new_subtree, "s", s, s->syn_oid);
 		    tmp = tmp->next;
 	       }
 	  }
@@ -295,6 +276,23 @@ void attach_server_schema(GtkWidget *item, GHashTable *hash)
 }
 
 
+/* more of a macro really, this makes inserting duplicates easier */
+void attach_schema_item(GtkWidget *tree, char *type, void *schemaobject, char *itemname)
+{
+     GtkWidget *new_item;
+
+     new_item = gtk_tree_item_new_with_label(itemname);
+     GTK_WIDGET_UNSET_FLAGS(new_item, GTK_CAN_FOCUS);
+     gtk_signal_connect(GTK_OBJECT(new_item), "button_press_event",
+			GTK_SIGNAL_FUNC(schema_button_tree), NULL);
+     gtk_object_set_data(GTK_OBJECT(new_item), type, schemaobject);
+     gtk_widget_show(new_item);
+     /* need gtk_tree_insert_sorted() here :-( */
+     gtk_tree_append(GTK_TREE(tree), new_item);
+
+}
+
+
 gboolean schema_button_tree(GtkWidget *tree_item, GdkEventButton *event)
 {
      GtkWidget *root_menu, *menu, *menu_item;
@@ -313,7 +311,7 @@ gboolean schema_button_tree(GtkWidget *tree_item, GdkEventButton *event)
 	  gtk_widget_show(menu_item);
 
 	  /* Refresh */
-	  menu_item = gtk_menu_item_new_with_label("Refresh");
+	  menu_item = gtk_menu_item_new_with_label(_("Refresh"));
 	  if(gtk_object_get_data(GTK_OBJECT(tree_item), "server") == NULL)
 	       gtk_widget_set_sensitive(menu_item, FALSE);
 	  gtk_menu_append(GTK_MENU(menu), menu_item);
@@ -322,7 +320,7 @@ gboolean schema_button_tree(GtkWidget *tree_item, GdkEventButton *event)
 			     GTK_SIGNAL_FUNC(schema_refresh_server), (gpointer) tree_item);
 
 	  /* Open in new window */
-	  menu_item = gtk_menu_item_new_with_label("Open in new window");
+	  menu_item = gtk_menu_item_new_with_label(_("Open in new window"));
 	  /* only leaf nodes can have a detail popup */
 	  if(GTK_TREE_ITEM(tree_item)->subtree)
 	       gtk_widget_set_sensitive(menu_item, FALSE);
@@ -411,7 +409,7 @@ void make_detail_notebook(GHashTable *hash)
      gtk_widget_show(dummy_vbox);
      gtk_box_pack_start(GTK_BOX(dummy_vbox), oc_vbox, TRUE, TRUE, 0);
      make_oc_detail(oc_vbox);
-     label = gtk_label_new("Objectclasses");
+     label = gtk_label_new(_("Objectclasses"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(rightpane_notebook), dummy_vbox, label);
 
@@ -420,7 +418,7 @@ void make_detail_notebook(GHashTable *hash)
      gtk_widget_show(dummy_vbox);
      gtk_box_pack_start(GTK_BOX(dummy_vbox), at_vbox, TRUE, TRUE, 0);
      make_at_detail(at_vbox);
-     label = gtk_label_new("Attribute types");
+     label = gtk_label_new(_("Attribute types"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(rightpane_notebook), dummy_vbox, label);
 
@@ -429,7 +427,7 @@ void make_detail_notebook(GHashTable *hash)
      gtk_widget_show(dummy_vbox);
      gtk_box_pack_start(GTK_BOX(dummy_vbox), mr_vbox, TRUE, TRUE, 0);
      make_mr_detail(mr_vbox);
-     label = gtk_label_new("Matching rules");
+     label = gtk_label_new(_("Matching rules"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(rightpane_notebook), dummy_vbox, label);
 
@@ -438,7 +436,7 @@ void make_detail_notebook(GHashTable *hash)
      gtk_widget_show(dummy_vbox);
      gtk_box_pack_start(GTK_BOX(dummy_vbox), s_vbox, TRUE, TRUE, 0);
      make_s_detail(s_vbox);
-     label = gtk_label_new("Syntaxes");
+     label = gtk_label_new(_("Syntaxes"));
      gtk_widget_show(label);
      gtk_notebook_append_page(GTK_NOTEBOOK(rightpane_notebook), dummy_vbox, label);
 
@@ -483,7 +481,7 @@ void popup_detail(enum schema_detail_type type, struct ldapserver *server, void
      GtkWidget *window, *vbox;
 
      window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-     gtk_window_set_title(GTK_WINDOW(window), "GQ");
+     gtk_window_set_title(GTK_WINDOW(window), _("GQ"));
      gtk_window_set_default_size(GTK_WINDOW(window), 446, 340);
      gtk_signal_connect_object(GTK_OBJECT(window), "key_press_event",
 			       (GtkSignalFunc) close_on_esc, (gpointer) window);
@@ -493,22 +491,22 @@ void popup_detail(enum schema_detail_type type, struct ldapserver *server, void
 
      switch(type) {
      case SCHEMA_TYPE_OC:
-	  gtk_window_set_title(GTK_WINDOW(window), "GQ: objectclass");
+	  gtk_window_set_title(GTK_WINDOW(window), _("GQ: objectclass"));
 	  make_oc_detail(vbox);
 	  fill_oc_detail(vbox, server, detail);
 	  break;
      case SCHEMA_TYPE_AT:
-	  gtk_window_set_title(GTK_WINDOW(window), "GQ: attribute type");
+	  gtk_window_set_title(GTK_WINDOW(window), _("GQ: attribute type"));
 	  make_at_detail(vbox);
 	  fill_at_detail(vbox, server, detail);
 	  break;
      case SCHEMA_TYPE_MR:
-	  gtk_window_set_title(GTK_WINDOW(window), "GQ: matching rule");
+	  gtk_window_set_title(GTK_WINDOW(window), _("GQ: matching rule"));
 	  make_mr_detail(vbox);
 	  fill_mr_detail(vbox, server, detail);
 	  break;
      case SCHEMA_TYPE_S:
-	  gtk_window_set_title(GTK_WINDOW(window), "GQ: syntax");
+	  gtk_window_set_title(GTK_WINDOW(window), _("GQ: syntax"));
 	  make_s_detail(vbox);
 	  fill_s_detail(vbox, server, detail);
 	  break;
@@ -523,8 +521,8 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
 {
      GtkWidget *label, *entry, *combo, *check, *clist;
      GtkWidget *hbox1, *hbox2, *vbox1, *table1, *scrwin, *paned;
-     char *rtitle[] = { "Required attributes", 0 };
-     char *atitle[] = { "Allowed attributes", 0 };
+     char *rtitle[] = { _("Required attributes"), 0 };
+     char *atitle[] = { _("Allowed attributes"), 0 };
 
      /* homogeneous hbox to split pane down the middle */
      hbox1 = gtk_hbox_new(TRUE, 0);
@@ -542,7 +540,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      gtk_box_pack_start(GTK_BOX(vbox1), table1, FALSE, FALSE, 0);
 
      /* Name */
-     label = gtk_label_new("Name");
+     label = gtk_label_new(_("Name"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -559,7 +557,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 1, 10);
 
      /* Description */
-     label = gtk_label_new("Description");
+     label = gtk_label_new(_("Description"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
@@ -575,7 +573,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 3, 10);
 
      /* OID */
-     label = gtk_label_new("OID");
+     label = gtk_label_new(_("OID"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 4, 5,
@@ -591,7 +589,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 5, 10);
 
      /* Superior */
-     label = gtk_label_new("Superior");
+     label = gtk_label_new(_("Superior"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 6, 7,
@@ -608,7 +606,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 7, 10);
 
      /* Kind */
-     label = gtk_label_new("Kind");
+     label = gtk_label_new(_("Kind"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 8, 9,
@@ -632,7 +630,7 @@ void make_oc_detail(GtkWidget *target_oc_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("Obsolete");
+     label = gtk_label_new(_("Obsolete"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 10, 11,
@@ -755,7 +753,7 @@ void fill_oc_detail(GtkWidget *target_oc_vbox, struct ldapserver *server,
 	  g_list_free(list);
      }
 
-     /* Kind */
+     /* Kind - peter: deliberaty no I18N (questionable) */
      entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(target_oc_vbox), "kind");
      switch(oc->oc_kind) {
      case LDAP_SCHEMA_ABSTRACT:
@@ -813,7 +811,7 @@ void make_at_detail(GtkWidget *target_vbox)
 {
      GtkWidget *hbox1, *hbox2, *vbox1, *vbox2, *table1, *table2, *scrwin;
      GtkWidget *label, *entry, *combo, *check, *clist;
-     char *otitle[] = { "Used in objectclasses", 0 };
+     char *otitle[] = { _("Used in objectclasses"), 0 };
 
      /* homogeneous hbox to split the pane down the middle */
      hbox1 = gtk_hbox_new(TRUE, 0);
@@ -832,7 +830,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_box_pack_start(GTK_BOX(vbox1), table1, FALSE, FALSE, 0);
 
      /* Name */
-     label = gtk_label_new("Name");
+     label = gtk_label_new(_("Name"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -849,7 +847,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 1, 10);
 
      /* Description */
-     label = gtk_label_new("Description");
+     label = gtk_label_new(_("Description"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
@@ -865,7 +863,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 3, 10);
 
      /* OID */
-     label = gtk_label_new("OID");
+     label = gtk_label_new(_("OID"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 4, 5,
@@ -881,7 +879,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 5, 10);
 
      /* Superior */
-     label = gtk_label_new("Superior");
+     label = gtk_label_new(_("Superior"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 6, 7,
@@ -897,7 +895,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 7, 10);
 
      /* Usage */
-     label = gtk_label_new("Usage");
+     label = gtk_label_new(_("Usage"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 8, 9,
@@ -921,7 +919,7 @@ void make_at_detail(GtkWidget *target_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("Obsolete");
+     label = gtk_label_new(_("Obsolete"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 10, 11,
@@ -936,7 +934,7 @@ void make_at_detail(GtkWidget *target_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("Single value");
+     label = gtk_label_new(_("Single value"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 11, 12,
@@ -951,7 +949,7 @@ void make_at_detail(GtkWidget *target_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("Collective");
+     label = gtk_label_new(_("Collective"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 12, 13,
@@ -967,7 +965,7 @@ void make_at_detail(GtkWidget *target_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("No user modification");
+     label = gtk_label_new(_("No user modification"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 13, 14,
@@ -986,7 +984,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_box_pack_start(GTK_BOX(vbox2), table2, FALSE, FALSE, 0);
 
      /* Equality */
-     label = gtk_label_new("Equality");
+     label = gtk_label_new(_("Equality"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, 0, 1,
@@ -1002,7 +1000,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table2), 1, 10);
 
      /* Ordering */
-     label = gtk_label_new("Ordering");
+     label = gtk_label_new(_("Ordering"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, 2, 3,
@@ -1018,7 +1016,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table2), 3, 10);
 
      /* Substrings */
-     label = gtk_label_new("Substrings");
+     label = gtk_label_new(_("Substrings"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, 4, 5,
@@ -1034,7 +1032,7 @@ void make_at_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table2), 5, 10);
 
      /* Syntax */
-     label = gtk_label_new("Syntax { length }");
+     label = gtk_label_new(_("Syntax { length }"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table2), label, 0, 1, 6, 7,
@@ -1132,7 +1130,7 @@ void fill_at_detail(GtkWidget *target_vbox, struct ldapserver *server,
      entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(target_vbox), "superior");
      gtk_entry_set_text(GTK_ENTRY(entry), at->at_sup_oid ? at->at_sup_oid : "");
 
-     /* Usage */
+     /* Usage - peter: deliberatly no I18N - questionable */
      entry = (GtkWidget *) gtk_object_get_data(GTK_OBJECT(target_vbox), "usage");
      switch(at->at_usage) {
      case LDAP_SCHEMA_USER_APPLICATIONS:
@@ -1218,7 +1216,7 @@ void make_mr_detail(GtkWidget *target_vbox)
 {
      GtkWidget *hbox1, *vbox1, *table1, *hbox2, *vbox2, *scrwin;
      GtkWidget *label, *combo, *entry, *check, *clist;
-     char *atitle[] = { "Used in attribute types", 0 };
+     char *atitle[] = { _("Used in attribute types"), 0 };
 
      /* homogeneous hbox to split the pane down the middle */
      hbox1 = gtk_hbox_new(TRUE, 0);
@@ -1237,7 +1235,7 @@ void make_mr_detail(GtkWidget *target_vbox)
      gtk_box_pack_start(GTK_BOX(vbox1), table1, FALSE, FALSE, 0);
 
      /* Name */
-     label = gtk_label_new("Name");
+     label = gtk_label_new(_("Name"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -1254,7 +1252,7 @@ void make_mr_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 1, 10);
 
      /* Description */
-     label = gtk_label_new("Description");
+     label = gtk_label_new(_("Description"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
@@ -1270,7 +1268,7 @@ void make_mr_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 3, 10);
 
      /* OID */
-     label = gtk_label_new("OID");
+     label = gtk_label_new(_("OID"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 4, 5,
@@ -1286,7 +1284,7 @@ void make_mr_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 5, 10);
 
      /* Syntax */
-     label = gtk_label_new("Syntax");
+     label = gtk_label_new(_("Syntax"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 6, 7,
@@ -1310,7 +1308,7 @@ void make_mr_detail(GtkWidget *target_vbox)
      GTK_WIDGET_UNSET_FLAGS(check, GTK_CAN_FOCUS);
      gtk_widget_show(check);
      gtk_box_pack_start(GTK_BOX(hbox2), check, FALSE, FALSE, 0);
-     label = gtk_label_new("Obsolete");
+     label = gtk_label_new(_("Obsolete"));
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
      gtk_table_attach(GTK_TABLE(table1), hbox2, 0, 1, 8, 9,
@@ -1437,8 +1435,8 @@ void make_s_detail(GtkWidget *target_vbox)
 {
      GtkWidget *hbox1, *vbox1, *table1, *scrwin, *paned;
      GtkWidget *label, *entry, *clist;
-     char *atitle[] = { "Used in attribute types", 0 };
-     char *mtitle[] = { "Used in matching rules", 0 };
+     char *atitle[] = { _("Used in attribute types"), 0 };
+     char *mtitle[] = { _("Used in matching rules"), 0 };
 
      /* homogeneous hbox to split the pane down the middle */
      hbox1 = gtk_hbox_new(TRUE, 0);
@@ -1457,7 +1455,7 @@ void make_s_detail(GtkWidget *target_vbox)
      gtk_box_pack_start(GTK_BOX(vbox1), table1, FALSE, FALSE, 0);
 
      /* OID */
-     label = gtk_label_new("OID");
+     label = gtk_label_new(_("OID"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 0, 1,
@@ -1473,7 +1471,7 @@ void make_s_detail(GtkWidget *target_vbox)
      gtk_table_set_row_spacing(GTK_TABLE(table1), 1, 10);
 
      /* Description */
-     label = gtk_label_new("Description");
+     label = gtk_label_new(_("Description"));
      gtk_misc_set_alignment(GTK_MISC(label), .0, .0);
      gtk_widget_show(label);
      gtk_table_attach(GTK_TABLE(table1), label, 0, 1, 2, 3,
