@@ -1,6 +1,6 @@
 /*
     GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2001 Bert Vermeulen
+    Copyright (C) 1998-2002 Bert Vermeulen
 
     This program is released under the Gnu General Public License with
     the additional exemption that compiling, linking, and/or using
@@ -21,6 +21,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/* $Id: search.c,v 1.22 2002/06/18 22:07:14 stamfest Exp $ */
 
 #include <config.h>
 
@@ -33,7 +34,6 @@
 #include "common.h"
 #include "mainwin.h"
 #include "configfile.h"
-#include "browse.h"         /* XXX for get_suffix(): move that to util.h */
 #include "util.h"
 #include "search.h"
 #include "formfill.h"
@@ -41,10 +41,15 @@
 #include "template.h"
 #include "tinput.h"
 #include "filter.h"
+#include "encode.h"
+#include "i18n.h"
 
 
 extern struct gq_config config;
 
+static gboolean search_button_press_on_tree_item(GtkWidget *clist,
+						 GdkEventButton *event,
+						 GHashTable *hash);
 
 void new_searchmode(GHashTable *hash)
 {
@@ -65,14 +70,14 @@ void new_searchmode(GHashTable *hash)
      searchtype_menu = gtk_menu_new();
      searchtype_om = gtk_option_menu_new();
 
-     searchtype_mi = gtk_radio_menu_item_new_with_label(NULL, LABEL_ST_SEARCH);
+     searchtype_mi = gtk_radio_menu_item_new_with_label(NULL, _("search"));
      gtk_object_set_data(GTK_OBJECT(searchtype_mi), "searchtype", GINT_TO_POINTER(ST_SEARCH));
      group = gtk_radio_menu_item_group(GTK_RADIO_MENU_ITEM(searchtype_mi));
      gtk_menu_append(GTK_MENU(searchtype_menu), searchtype_mi);
      gtk_widget_show(searchtype_mi);
      g_hash_table_insert(hash, "searchtype_item", searchtype_mi);
 
-     searchtype_mi = gtk_radio_menu_item_new_with_label(group, LABEL_ST_FILTER);
+     searchtype_mi = gtk_radio_menu_item_new_with_label(group, _("filter"));
      gtk_object_set_data(GTK_OBJECT(searchtype_mi), "searchtype", GINT_TO_POINTER(ST_FILTER));
      gtk_menu_append(GTK_MENU(searchtype_menu), searchtype_mi);
      gtk_widget_show(searchtype_mi);
@@ -123,7 +128,7 @@ void new_searchmode(GHashTable *hash)
      g_hash_table_insert(hash, "searchbase_combo", searchbase_combo);
 
      /* find button */
-     findbutton = gtk_button_new_with_label(" Find ");
+     findbutton = gtk_button_new_with_label(_(" Find "));
      GTK_WIDGET_UNSET_FLAGS(findbutton, GTK_CAN_FOCUS);
      gtk_widget_show(findbutton);
      gtk_box_pack_start(GTK_BOX(hbox1), findbutton, FALSE, TRUE, 6);
@@ -164,6 +169,7 @@ void servername_changed_callback(GHashTable *hash)
      servcombo = g_hash_table_lookup(hash, "serverlist_combo");
      cur_servername = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(servcombo)->entry), 0, -1);
      cur_server = server_by_name(cur_servername);
+     g_free(cur_servername);
      g_hash_table_insert(hash, "cur_server", cur_server);
 
      /* make sure searchbase gets refreshed next time the searchbase combo button is
@@ -174,6 +180,7 @@ void servername_changed_callback(GHashTable *hash)
 	  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(searchbase_combo)->entry), cur_server->basedn);
 	  searchbase_list = g_list_append(NULL, cur_server->basedn);
 	  gtk_combo_set_popdown_strings(GTK_COMBO(searchbase_combo), searchbase_list);
+	  g_list_free(searchbase_list);
      }
 
 }
@@ -182,7 +189,7 @@ void servername_changed_callback(GHashTable *hash)
 gint searchbase_button_pressed(GtkWidget *widget, GdkEventButton *event, GHashTable *hash)
 {
      GList *searchbase_list;
-     GSList *suffixes_list, *temp;
+     GList *suffixes_list, *temp;
      GtkWidget *servcombo, *searchbase_combo;
      struct ldapserver *server;
      int found_default_searchbase;
@@ -195,18 +202,19 @@ gint searchbase_button_pressed(GtkWidget *widget, GdkEventButton *event, GHashTa
 	  servcombo = g_hash_table_lookup(hash, "serverlist_combo");
 	  cur_servername = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(servcombo)->entry), 0, -1);
 	  server = server_by_name(cur_servername);
+	  g_free(cur_servername);
 	  if (!server)
 	       return(FALSE);
 
 	  searchbase_list = NULL;
-	  suffixes_list = get_suffix(server);
+	  suffixes_list = get_suffixes(server);
     
 	  temp = suffixes_list;
 	  while (temp) {
 	       if (g_strcasecmp(server->basedn, temp->data) == 0)
 		    found_default_searchbase = 1;
 	       searchbase_list = g_list_append(searchbase_list, temp->data);
-      	       temp = g_slist_next(temp);
+      	       temp = g_list_next(temp);
 	  }
 
 	  if (found_default_searchbase == 0)
@@ -218,10 +226,10 @@ gint searchbase_button_pressed(GtkWidget *widget, GdkEventButton *event, GHashTa
 	  temp = suffixes_list;
 	  while (temp) {
 	       g_free(temp->data);
-	       temp = g_slist_next(temp);
+	       temp = g_list_next(temp);
 	  }
 	  if (suffixes_list)
-	       g_slist_free(suffixes_list);
+	       g_list_free(suffixes_list);
 	  g_list_free(searchbase_list);
 
 	  g_hash_table_insert(hash, "populated_searchbase", GINT_TO_POINTER(1));
@@ -356,7 +364,7 @@ void query(GHashTable *hash)
      GSList *group;
      LDAP *ld;
      LDAPMessage *res, *e;
-     BerElement *ptr;
+     BerElement *berptr;
      GtkWidget *main_clist, *new_main_clist, *scrwin, *focusbox;
      GtkWidget *servcombo, *searchbase_combo, *searchtype_item;
      GtkRadioMenuItem *menuitem;
@@ -364,8 +372,9 @@ void query(GHashTable *hash)
      struct resultset *set, *cur_resultset;
      struct attrs *attrlist;
      gchar *cur_servername, *cur_searchbase, *querystring, *cl[MAX_NUM_ATTRIBUTES];
-     char tolist[MAX_NUM_ATTRIBUTES][128], message[1152], filter[MAX_LDAPFILTER_LEN];
-     char *attr, *dn, **vals;
+     char tolist[MAX_NUM_ATTRIBUTES][128], message[MAX_LDAPFILTER_LEN + 128];
+     char filter[MAX_LDAPFILTER_LEN];
+     char *attr, *dn, **vals, *searchterm, *decoded_value;
      int searchtype, msg, rc, i, row;
      int cur_col, oc_col, columns_done[MAX_NUM_ATTRIBUTES];
 
@@ -375,7 +384,9 @@ void query(GHashTable *hash)
      g_hash_table_insert(hash, "search lock", "ongoing");
 
      focusbox = g_hash_table_lookup(hash, "focus");
-     querystring = gtk_editable_get_chars(GTK_EDITABLE(focusbox), 0, -1);
+     searchterm = gtk_editable_get_chars(GTK_EDITABLE(focusbox), 0, -1);
+     querystring = encoded_string(searchterm);
+     g_free(searchterm);
 
      for(i = 0; i < MAX_NUM_ATTRIBUTES; i++)
 	  columns_done[i] = 0;
@@ -383,6 +394,7 @@ void query(GHashTable *hash)
      servcombo = g_hash_table_lookup(hash, "serverlist_combo");
      cur_servername = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(servcombo)->entry), 0, -1);
      server = server_by_name(cur_servername);
+     g_free(cur_servername);
      if(!server) {
 	  statusbar_msg("Oops! No server found!");
 	  g_hash_table_remove(hash, "search lock");
@@ -436,9 +448,9 @@ void query(GHashTable *hash)
 	  break;
      };
 
-     g_free(querystring);
+     free(querystring);
 
-     sprintf(message, "searching for %s", filter);
+     snprintf(message, sizeof(message), "searching for %s", filter);
      statusbar_msg(message);
 
      searchbase_combo = g_hash_table_lookup(hash, "searchbase_combo");
@@ -449,10 +461,15 @@ void query(GHashTable *hash)
      else
 	  msg = ldap_search(ld, cur_searchbase, LDAP_SCOPE_SUBTREE,
 			    filter, NULL, 0);
+     g_free(cur_searchbase);
 
      if((config.sort_search && msg != LDAP_SUCCESS) || msg == -1) {
+	  if (config.sort_search && msg != LDAP_SERVER_DOWN) {
+	       server->server_down++;
+	  }
 	  statusbar_msg(ldap_err2string(msg == -1 ? LDAP_OTHER : msg));
 	  g_hash_table_remove(hash, "search lock");
+	  close_connection(server, FALSE);
 	  return;
      }
 
@@ -473,9 +490,12 @@ void query(GHashTable *hash)
      gtk_signal_connect(GTK_OBJECT(new_main_clist), "select_row",
                         GTK_SIGNAL_FUNC(select_entry_callback),
                         hash);
-     gtk_signal_connect(GTK_OBJECT(new_main_clist), "unselect_row",
-                        GTK_SIGNAL_FUNC(unselect_entry_callback),
-                        hash);
+/*       gtk_signal_connect(GTK_OBJECT(new_main_clist), "unselect_row", */
+/*                          GTK_SIGNAL_FUNC(unselect_entry_callback), */
+/*                          hash); */
+     gtk_signal_connect(GTK_OBJECT(new_main_clist), "button_press_event",
+			GTK_SIGNAL_FUNC(search_button_press_on_tree_item),
+			hash);
 
      main_clist = g_hash_table_lookup(hash, "main_clist");
      gtk_clist_clear(GTK_CLIST(main_clist));
@@ -524,7 +544,9 @@ void query(GHashTable *hash)
 	       strncpy(set->dn, dn, MAX_DN_LEN);
 
 	       if(config.showdn) {
-		    strcpy(tolist[0],dn);
+		    decoded_value = decoded_string(dn);
+		    strcpy(tolist[0], decoded_value);
+		    free(decoded_value);
 		    cl[0] = tolist[0];
 	       }
 #if defined(HAVE_LDAP_MEMFREE)
@@ -533,15 +555,19 @@ void query(GHashTable *hash)
                free(dn);
 #endif
 
-	       for(attr = ldap_first_attribute(ld, e, &ptr); attr != NULL;
-		   attr = ldap_next_attribute(ld, e, ptr)) {
-
+	       for(attr = ldap_first_attribute(ld, e, &berptr); attr != NULL;
+		   attr = ldap_next_attribute(ld, e, berptr)) {
+		    /* This should now work for ;binary as well */
 		    cur_col = column_by_attr(&attrlist, attr);
-		    if(cur_col == MAX_NUM_ATTRIBUTES)
+		    if(cur_col == MAX_NUM_ATTRIBUTES) {
+			 ldap_memfree(attr);
 			 break;
+		    }
 
 		    if(!columns_done[cur_col]) {
-			 gtk_clist_set_column_title(GTK_CLIST(new_main_clist), cur_col, attr);
+			 char *c = attr_strip(attr);
+			 gtk_clist_set_column_title(GTK_CLIST(new_main_clist), cur_col, c);
+			 if (c) g_free(c);
 			 gtk_clist_set_column_width(GTK_CLIST(new_main_clist), cur_col, 120);
 			 columns_done[cur_col] = 1;
 		    }
@@ -549,19 +575,26 @@ void query(GHashTable *hash)
 		    vals = ldap_get_values(ld, e, attr);
 		    if(vals) {
 			 for(i = 0; vals[i] != NULL; i++) {
+			      /* that's an awful lot of mallocs in the innermost loop,
+				 but then we don't want to count on max valuelen either */
+			      decoded_value = decoded_string(vals[i]);
 			      if(i == 0)
-				   strcpy(tolist[cur_col], vals[i]);
+				   strcpy(tolist[cur_col], decoded_value);
 			      else {
 				   strcat(tolist[cur_col], " ");
-				   strcat(tolist[cur_col], vals[i]);
+				   strcat(tolist[cur_col], decoded_value);
 			      }
-
+			      free(decoded_value);
 			 }
 			 ldap_value_free(vals);
 			 cl[cur_col] = tolist[cur_col];
 		    }
-
+		    ldap_memfree(attr);
 	       }
+#ifndef HAVE_OPENLDAP12
+	       if(berptr)
+		    ber_free(berptr, 0);
+#endif
 
 	       /* insert row into result window */
 	       gtk_clist_append(GTK_CLIST(new_main_clist), cl);
@@ -584,7 +617,8 @@ void query(GHashTable *hash)
      if(rc == -1)
 	  statusbar_msg(ldap_err2string(msg));
      else {
-	  make_message(message, row, "entry", "entries", "found");
+	  make_message(message, sizeof(message), 
+		       row, "entry", "entries", "found");
 	  statusbar_msg(message);
      }
 
@@ -618,7 +652,7 @@ void results_popup_menu(GHashTable *hash, GdkEventButton *event, struct resultse
      gtk_widget_show(menu_item);
 
      /* Edit */
-     menu_item = gtk_menu_item_new_with_label("Edit");
+     menu_item = gtk_menu_item_new_with_label(_("Edit"));
      gtk_menu_append(GTK_MENU(menu), menu_item);
      gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
 			       GTK_SIGNAL_FUNC(search_edit_entry_callback),
@@ -626,7 +660,7 @@ void results_popup_menu(GHashTable *hash, GdkEventButton *event, struct resultse
      gtk_widget_show(menu_item);
 
      /* Use as template */
-     menu_item = gtk_menu_item_new_with_label("Use as template");
+     menu_item = gtk_menu_item_new_with_label(_("Use as template"));
      gtk_menu_append(GTK_MENU(menu), menu_item);
      gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
 			       GTK_SIGNAL_FUNC(search_new_from_entry_callback),
@@ -639,7 +673,7 @@ void results_popup_menu(GHashTable *hash, GdkEventButton *event, struct resultse
      gtk_widget_show(menu_item);
 
      /* Delete */
-     menu_item = gtk_menu_item_new_with_label("Delete");
+     menu_item = gtk_menu_item_new_with_label(_("Delete"));
      gtk_menu_append(GTK_MENU(menu), menu_item);
      gtk_signal_connect_object(GTK_OBJECT(menu_item), "activate",
 			       GTK_SIGNAL_FUNC(delete_search_entry),
@@ -713,23 +747,18 @@ int select_entry_callback(GtkWidget *clist, gint row, gint column,
      struct resultset *set;
 
      if(event) {
-
 	  set = gtk_clist_get_row_data(GTK_CLIST(clist), row);
 
 	  if(event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
 	       g_hash_table_insert(hash, "set", set);
 	       search_edit_entry_callback(hash);
 	  }
-	  else if(event->button == 3) {
-	       results_popup_menu(hash, event, set);
-	  }
-
      }
 
      return(TRUE);
 }
 
-
+#if 0   /* no longer needed */
 int unselect_entry_callback(GtkWidget *clist, gint row, gint column,
 			   GdkEventButton *event, gpointer hash)
 {
@@ -753,6 +782,31 @@ int unselect_entry_callback(GtkWidget *clist, gint row, gint column,
 
      return(TRUE);
 }
+#endif
+
+static gboolean search_button_press_on_tree_item(GtkWidget *clist,
+						 GdkEventButton *event,
+						 GHashTable *hash) 
+{
+     struct resultset *set;
+     int rc, row, column;
+
+     if (event->type == GDK_BUTTON_PRESS && event->button == 3
+	 && event->window == GTK_CLIST(clist)->clist_window) {
+	  rc = gtk_clist_get_selection_info(GTK_CLIST(clist), 
+					    event->x, event->y, &row, &column);
+	  if (rc == 0)
+	       return TRUE;
+
+	  set = gtk_clist_get_row_data(GTK_CLIST(clist), row);
+	  results_popup_menu(hash, event, set);
+
+	  gtk_signal_emit_stop_by_name(GTK_OBJECT(clist),
+				       "button_press_event");
+     }
+
+     return TRUE;
+}
 
 
 void cleanup_search_mode(GHashTable *hash)
@@ -773,3 +827,9 @@ void cleanup_search_mode(GHashTable *hash)
 
 }
 
+
+/* 
+   Local Variables:
+   c-basic-offset: 5
+   End:
+ */
