@@ -1,6 +1,6 @@
 /*
     GQ -- a GTK-based LDAP client
-    Copyright (C) 1998-2001 Bert Vermeulen
+    Copyright (C) 1998-2002 Bert Vermeulen
 
     This program is released under the Gnu General Public License with
     the additional exemption that compiling, linking, and/or using
@@ -42,6 +42,7 @@
 #include "schema.h"
 #include "schemabrowse.h"
 #include "template.h"
+#include "tdefault.h"
 #include "errorchain.h"
 #include "debug.h"
 
@@ -58,7 +59,7 @@ void create_template_edit_window(struct ldapserver *server, char *templatename)
      GList *list;
      GString *gmessage;
      GtkWidget *vbox1, *hbox0, *hbox1, *vbox2, *vbox3, *vbox4;
-     GtkWidget *hbox3, *hbox4, *scrwin;
+     GtkWidget *hbox2, *hbox3, *hbox4, *scrwin;
      GtkWidget *oclist, *templatelist, *clist, *button, *arrow, *label, *entry;
      struct server_schema *ss;
      struct gq_template *tmpl;
@@ -221,6 +222,22 @@ void create_template_edit_window(struct ldapserver *server, char *templatename)
      gtk_container_add(GTK_CONTAINER(scrwin), clist);
      gtk_box_pack_start(GTK_BOX(vbox4), scrwin, TRUE, TRUE, 0);
 
+     hbox2 = gtk_hbox_new(FALSE, 0);
+     gtk_widget_show(hbox2);
+     gtk_box_pack_start(GTK_BOX(vbox1), hbox2, FALSE, FALSE, 10);
+
+#if EXPERIMENTAL_TEMPLATE_DEFAULTS
+     /* Input defaults */
+     button = gtk_button_new_with_label("  Edit input defaults  ");
+     GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_DEFAULT);
+     GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+     gtk_signal_connect(GTK_OBJECT(button), "pressed",
+			(GtkSignalFunc) create_tdefault_edit_window,
+			(gpointer) templatewin);
+     gtk_widget_show(button);
+     gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
+#endif
+
      hbox3 = gtk_hbox_new(FALSE, 0);
      gtk_widget_show(hbox3);
      gtk_box_pack_start(GTK_BOX(vbox1), hbox3, FALSE, FALSE, 10);
@@ -387,7 +404,7 @@ void arrow_button_callback(GtkWidget *widget, GtkWidget *window)
 
 /*
  * default clist sort function is case-sensitive, this does a case-insensitive
- * search instead
+ * sort instead
  */
 int clistcasecmp(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
 {
